// ============================================================================
// Copyright (c) 2022 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Fri Jul 15 14:16:19 2022
// ============================================================================

//`define ENABLE_DDR4A
//`define ENABLE_DDR4B
//`define ENABLE_QSFP28
//`define ENABLE_FMCP
//`define ENABLE_HPS
//`define ENABLE_HDMI
//`define ENABLE_PCIE

module golden_top(

      ///////// CLOCK /////////
      input              CLK_100_B3A_p,
      input              CLK_100_p,
      input              CLK_148M5_p,
      input              CLK_50_B2D_p,
      input              CLK_50_B2E_p,
      input              CLK_50_B3A_p,
      input              CLK_50_B3C_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

		
`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output             DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output             DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output             DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 0: 0]   DDR4B_BG,
      output             DDR4B_CK,
      output             DDR4B_CK_n,
      output             DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output             DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output             DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      output   [ 3: 0]   HDMI_TX_D_p,
      output   [ 3: 0]   HDMI_TX_D_n,
      input              HDMI_REFCLK_100_p,
      input              HDMI_REFCLK_148M5_p,
      output             DVI_TX_5V,
      input              DVI_TX_CEC_IN_n,
      output             DVI_TX_CEC_OUT_n,
      input              DVI_TX_HPD_n,
      inout              DVI_TX_SCL,
      inout              DVI_TX_SDA,
      inout              HDMI_TX_SCL,
      inout              HDMI_TX_SDA,
`endif /*ENABLE_HDMI*/

      ///////// SI5340A0 /////////
      inout              SI5340A0_I2C_SCL,
      inout              SI5340A0_I2C_SDA,
      output             SI5340A0_OE_n,
      output             SI5340A0_RST_n,

      ///////// SI5340A1 /////////
      inout              SI5340A1_I2C_SCL,
      inout              SI5340A1_I2C_SDA,
      output             SI5340A1_OE_n,
      output             SI5340A1_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_P,

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      inout              FMCP_CLK2_BIDIR_p,
      inout              FMCP_CLK2_BIDIR_n,
      inout              FMCP_CLK3_BIDIR_p,
      inout              FMCP_CLK3_BIDIR_n,
      output   [15: 0]   FMCP_DP_C2M_p,
      output   [15: 0]   FMCP_DP_C2M_n,
      input    [15: 0]   FMCP_DP_M2C_p,
      input    [15: 0]   FMCP_DP_M2C_n,
      inout    [23: 0]   FMCP_HA_p,
      inout    [23: 0]   FMCP_HA_n,
      inout    [21: 0]   FMCP_HB_p,
      inout    [21: 0]   FMCP_HB_n,
      inout    [33: 0]   FMCP_LA_p,
      inout    [33: 0]   FMCP_LA_n,
      input    [ 1: 0]   FMCP_CLK_M2C_p,
      input    [ 1: 0]   FMCP_GBTCLK_M2C_p,
      output             FMCP_REFCLK_C2M_p,
      input              FMCP_REFCLK_M2C_p,
      input    [ 3: 0]   FMCP_REFCLK_p,
      inout    [ 1: 0]   FMCP_GA,
      input              FMCP_RES0,
      inout              FMCP_RES1,
      inout              FMCP_SCL,
      inout              FMCP_SDA,
      output             FMCP_SYNC_C2M_p,
      input              FMCP_SYNC_M2C_p,
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      inout              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output   [ 7: 0]   PCIE_TX_p,
      output   [ 7: 0]   PCIE_TX_n,
      input    [ 7: 0]   PCIE_RX_p,
      input    [ 7: 0]   PCIE_RX_n,
      input              PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFP28
      ///////// QSFP28 /////////
      output   [ 3: 0]   QSFP28_TX_p,
      output   [ 3: 0]   QSFP28_TX_n,
      input    [ 3: 0]   QSFP28_RX_p,
      input    [ 3: 0]   QSFP28_RX_n,
      input    [ 1: 0]   QSFP28_REFCLK_p,
      input              QSFP28_INTERRUPT_n,
      output             QSFP28_LP_MODE,
      input              QSFP28_MOD_PRS_n,
      output             QSFP28_MOD_SEL_n,
      output             QSFP28_RST_n,
      inout              QSFP28_SCL,
      inout              QSFP28_SDA,
`endif /*ENABLE_QSFP28*/

      ///////// EXP /////////
      input              EXP_EN,

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      input              HPS_JTAG_TCK,
      input              HPS_JTAG_TDI,
      output             HPS_JTAG_TDO,
      input              HPS_JTAG_TMS,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
`endif /*ENABLE_HPS*/


      input    [ 1: 0]   FMCP_REFCLK_p,  //0:default 184.32MHZ Si5340_1_OUT1 ,1:default 156.25MHZ  Si5340_1_OUT0
      input    [ 1: 0]   QSFP28_REFCLK_p,//0:default 156.25MHZ Si5340_0_OUT0 ,1:default 184.32MHZ  Si5340_1_OUT1		

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n
		

);

//=======================================================
//  REG/WIRE declarations
//=======================================================
wire [7:0]  HZ; 
wire        M50_1HZ ; 
wire        si5340a_controller_start;
wire        [1:0]si5340a_config_done;
wire        [1:0]tx_clkout_FMCP_REFCLK_p;
wire        [1:0]tx_clkout_QSFP28_REFCLK_p;
wire        reset_n ; 
wire        RESET_N_DY ; 
//=======================================================
//  Structural coding
//=======================================================

//----reset 
assign CPU_RESET_n    =1'bz ; 
//============== reset  =================================
RESET_DELY RESET_DELY_inst( .OSC_50(CLK_50_B2D_p) , .RESET_N_DY(RESET_N_DY) ) ;
assign reset_n = BUTTON[0] & RESET_N_DY;

//=======================================================
//  Configure SI5340A 
//=======================================================


//----- Configure SI5340A 
`define SI5340A_FREQ_644M53125     4'h0  
`define SI5340A_FREQ_625M          4'h1  
`define SI5340A_FREQ_322M265625    4'h2  
`define SI5340A_FREQ_312M5         4'h3  
`define SI5340A_FREQ_250M          4'h4  
`define SI5340A_FREQ_184M32        4'h5  
`define SI5340A_FREQ_156M25        4'h6  
`define SI5340A_FREQ_148M5         4'h7  
`define SI5340A_FREQ_125M          4'h8  
`define SI5340A_FREQ_100M          4'h9  

//======start config ======================================
assign si5340a_controller_start = ~BUTTON[0]| ~reset_n;

//============= reconfig Si5340A (U77) ====================
//QSFP28_REFCLK_p[0]:default 156.25MHZ  Si5340_0_OUT0 
//QSFP28_REFCLK_p[1]:default 184.32MHZ  Si5340_0_OUT1
DE10AGILEX_SI5340A_CONFIG si5340a_controller_a0(
    .iCLK                (CLK_50_B2D_p),
    .iRST_n              (1),
    .iStart              (si5340a_controller_start),
    .iFREQ_OUT_0         ( `SI5340A_FREQ_156M25 ),//QSFP28_REFCLK_p0(to FGT PLL) , specify desired out frequency for Si5340 output port 0 by constant SI5340A_FREQ_1_XXX 
    .iFREQ_OUT_1         ( `SI5340A_FREQ_184M32 ),//QSFP28_REFCLK_p1(to FGT PLL), specify desired out frequency for Si5340 output port 1 by constant SI5340A_FREQ_1_XXX 
	 .iFREQ_OUT_3         ( `SI5340A_FREQ_148M5  ),//CLKGEN_100_p    (to IO PLL ), specify desired out frequency for Si5340 output port 3 by constant SI5340A_FREQ_1_XXX 
    .I2C_CLK             (SI5340A0_I2C_SCL),
    .I2C_DATA            (SI5340A0_I2C_SDA),
    .oPLL_REG_CONFIG_DONE  ( si5340a_config_done[0])
);
assign SI5340A0_OE_n  = 1'b0;
assign SI5340A0_RST_n = 1'b1;


//============= reconfig Si5340 (U68)============================
//FMCP_REFCLK_p[1]:default 156.25MHZ  Si5340_1_OUT0
//FMCP_REFCLK_p[0]:default 184.32MHZ  Si5340_1_OUT1 
DE10AGILEX_SI5340A_CONFIG si5340a_controller_a1(
    .iCLK                (CLK_50_B2D_p),
    .iRST_n              (1),
    .iStart              (si5340a_controller_start),
    .iFREQ_OUT_0         ( `SI5340A_FREQ_156M25 ),//FMCP_REFCLK1_p (to FGT PLL) , specify desired out frequency for Si5340 output port 0 by constant SI5340A_FREQ_1_XXX 
    .iFREQ_OUT_1         ( `SI5340A_FREQ_184M32 ),//FMCP_REFCLK0_p (to FGT PLL), specify desired out frequency for Si5340 output port 1 by constant SI5340A_FREQ_1_XXX 
	 .iFREQ_OUT_3         ( `SI5340A_FREQ_148M5  ),//CLKGEN_148M5_p (to IO PLL ), specify desired out frequency for Si5340 output port 3 by constant SI5340A_FREQ_1_XXX 
    .I2C_CLK             (SI5340A1_I2C_SCL),
    .I2C_DATA            (SI5340A1_I2C_SDA),
    .oPLL_REG_CONFIG_DONE  ( si5340a_config_done[1])
);
assign SI5340A1_OE_n  = 1'b0;
assign SI5340A1_RST_n = 1'b1;

//============= heart  ==========================================
CLOCKMEM  HZ_CLK_50_B3C_p               ( .RESET_n(reset_n  ) ,.CLK( CLK_50_B3C_p                 ),.CLK_FREQ   ( 50_000_000 )  ,.CK_1HZ( HZ[0] ) ) ;
CLOCKMEM  HZ_CLK_148M5_p                ( .RESET_n(reset_n  ) ,.CLK( CLK_148M5_p                  ),.CLK_FREQ   ( 148_500_000)  ,.CK_1HZ( HZ[1] ) ) ;
CLOCKMEM  HZ_tx_clkout_FMCP_REFCLK_p0   ( .RESET_n(reset_n  ) ,.CLK( tx_clkout_FMCP_REFCLK_p[0]   ),.CLK_FREQ   ( 184_320_000)  ,.CK_1HZ( HZ[2] ) ) ;
CLOCKMEM  HZ_tx_clkout_FMCP_REFCLK_p1   ( .RESET_n(reset_n  ) ,.CLK( tx_clkout_FMCP_REFCLK_p[1]   ),.CLK_FREQ   ( 156_250_000)  ,.CK_1HZ( HZ[3] ) ) ;
CLOCKMEM  HZ_tx_clkout_QSFP28_REFCLK_p0 ( .RESET_n(reset_n  ) ,.CLK( tx_clkout_QSFP28_REFCLK_p[0] ),.CLK_FREQ   ( 156_250_000)  ,.CK_1HZ( HZ[4] ) ) ;
CLOCKMEM  HZ_tx_clkout_QSFP28_REFCLK_p1 ( .RESET_n(reset_n  ) ,.CLK( tx_clkout_QSFP28_REFCLK_p[1] ),.CLK_FREQ   ( 184_320_000)  ,.CK_1HZ( HZ[5] ) ) ;
CLOCKMEM  HZ_CLK_100_p                  ( .RESET_n(reset_n  ) ,.CLK( CLK_100_p ),                   .CLK_FREQ   ( 148_500_000)  ,.CK_1HZ( HZ[6] ) ) ;

//============= LED Display =====================================
//SW0=1: see U68 out1/out3 heart  , SW0=0: see U77 out1/out3 heart  . 
assign LED         = &si5340a_config_done ? ( SW[0]? { HZ[2] , HZ[1] } : { HZ[6] , HZ[5] } ) : 2'b00 ;

//============= REFCLK Measure , view SignalTapeII ==============
//--1hz generater 
CLOCKMEM  ck2( .RESET_n(reset_n), .CLK(CLK_50_B3C_p) ,.CLK_FREQ ( 50000000 )  ,.CK_1HZ( M50_1HZ ) ) ;


//============= view frequency by SignalTapeII ==================
FRE_MEASURE ftx_clkout_FMCP_REFCLK_p0  ( .RESET_n(reset_n),.MES_CLK (tx_clkout_FMCP_REFCLK_p[0])   , .HZ1(M50_1HZ) , .FREQ( ) ); 
FRE_MEASURE ftx_clkout_FMCP_REFCLK_p1  ( .RESET_n(reset_n),.MES_CLK (tx_clkout_FMCP_REFCLK_p[1])   , .HZ1(M50_1HZ) , .FREQ( ) ); 
FRE_MEASURE ftx_clkout_QSFP28_REFCLK_p0( .RESET_n(reset_n),.MES_CLK (tx_clkout_QSFP28_REFCLK_p[0])   , .HZ1(M50_1HZ) , .FREQ( ) ); 
FRE_MEASURE ftx_clkout_QSFP28_REFCLK_p1( .RESET_n(reset_n),.MES_CLK (tx_clkout_QSFP28_REFCLK_p[1])   , .HZ1(M50_1HZ) , .FREQ( ) ); 
FRE_MEASURE fCLK_148M5_p               ( .RESET_n(reset_n),.MES_CLK (CLK_148M5_p  )                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 
FRE_MEASURE fCLK_100_B3A_p             ( .RESET_n(reset_n),.MES_CLK (CLK_100_B3A_p)                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 
FRE_MEASURE fCLK_100_p                 ( .RESET_n(reset_n),.MES_CLK (CLK_100_p    )                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 
FRE_MEASURE fCLK_50_B2D_p              ( .RESET_n(reset_n),.MES_CLK (CLK_50_B2D_p )                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 
FRE_MEASURE fCLK_50_B2E_p              ( .RESET_n(reset_n),.MES_CLK (CLK_50_B2E_p )                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 
FRE_MEASURE fCLK_50_B3A_p              ( .RESET_n(reset_n),.MES_CLK (CLK_50_B3A_p )                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 
FRE_MEASURE fCLK_50_B3C_p              ( .RESET_n(reset_n),.MES_CLK (CLK_50_B3C_p )                  , .HZ1(M50_1HZ) , .FREQ(  ) ); 


//============== bypass FMCP_REFCLK_p to user logic =============
wire fmcp_systempll_clk,fmcp_refclk_fgt_3;
pllsys_fmcp pllsys_inst0(
		.out_systempll_synthlock_0(), 
		.out_systempll_clk_0      (fmcp_systempll_clk),       
		.out_refclk_fgt_2         (),          
		.out_refclk_fgt_3         (fmcp_refclk_fgt_3  ),          
		.in_refclk_fgt_2          (FMCP_REFCLK_p[0]   ),//184.32 //ch2           
		.in_refclk_fgt_3          (FMCP_REFCLK_p[1]   ),//156.25 //ch3                    
		.out_coreclk_2            (tx_clkout_FMCP_REFCLK_p[0]), 
		.out_coreclk_3            (tx_clkout_FMCP_REFCLK_p[1])  
	);
wire fmcp_tx_clkout;	
phy_fmcp phy_fmcp_inst0(
		/*input  wire       */.tx_pll_refclk_link (fmcp_refclk_fgt_3),  
		/*input  wire [0:0] */.system_pll_clk_link(fmcp_systempll_clk), 
		/*input  wire [0:0] */.tx_reset(),                              
		/*output wire [0:0] */.tx_reset_ack(),                          
		/*output wire [0:0] */.tx_ready(),                              
		/*input  wire       */.tx_coreclkin(fmcp_tx_clkout),            
		/*output wire       */.tx_clkout (fmcp_tx_clkout),              
		/*output wire [0:0] */.tx_serial_data(),                        
		/*output wire [0:0] */.tx_serial_data_n(),                      
		/*output wire [0:0] */.tx_pll_locked(),                         
		/*input  wire [79:0]*/.tx_parallel_data()                      
	);
	
//========== bypass QSFP28_REFCLK_p to user logic ====================
wire qsfp28_systempll_clk,qsfp28_refclk_fgt_4;	
pllsys_qsfp28 pllsys_qsfp28_inst0(
		.out_systempll_synthlock_0(), 
		.out_systempll_clk_0      (qsfp28_systempll_clk),       
		.out_refclk_fgt_4         (qsfp28_refclk_fgt_4),          
		.out_refclk_fgt_5         (),          
		.in_refclk_fgt_4          (QSFP28_REFCLK_p[0]),   //156.25  ch4
		.in_refclk_fgt_5          (QSFP28_REFCLK_p[1]),   //184.32  ch5                 
		.out_coreclk_4            (tx_clkout_QSFP28_REFCLK_p[0]),             
		.out_coreclk_5            (tx_clkout_QSFP28_REFCLK_p[1])  
	);
wire qsfp28_tx_clkout;	
phy_qsfp28 phy_qsfp28_inst0(
		/*input  wire       */.tx_pll_refclk_link (qsfp28_refclk_fgt_4), 
		/*input  wire [0:0] */.system_pll_clk_link(qsfp28_systempll_clk),
		/*input  wire [0:0] */.tx_reset(),                               
		/*output wire [0:0] */.tx_reset_ack(),                           
		/*output wire [0:0] */.tx_ready(),                               
		/*input  wire       */.tx_coreclkin(qsfp28_tx_clkout),           
		/*output wire       */.tx_clkout   (qsfp28_tx_clkout),           
		/*output wire [0:0] */.tx_serial_data(),                        
		/*output wire [0:0] */.tx_serial_data_n(),                      
		/*output wire [0:0] */.tx_pll_locked(),                         
		/*input  wire [79:0]*/.tx_parallel_data()                      
	);

endmodule
