// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// Clock Monitors

module eth_f_clock_mon
  #(
    parameter SIM_HURRY   = 1'b0,
    parameter SIM_EMULATE = 1'b0
    )
   
   (
    input wire 	      csr_clk,
    input wire 	      clk_tx,
    input wire 	      clk_rx,
    input wire 	      clk_pll,
    input wire 	      clk_tx_div,
    input wire 	      clk_rec_div64,
    input wire 	      clk_rec_div,

    output reg [31:0] clk_tx_khz,
    output reg [31:0] clk_rx_khz,
    output reg [31:0] clk_pll_khz,
    output reg [31:0] clk_tx_div_khz,
    output reg [31:0] clk_rec_div64_khz,
    output reg [31:0] clk_rec_div_khz          
    
    );
   
//// output mapping is as follows - 000		, 101			,100		,011	,010	,001	   
wire [7:0] mon_clocks = {2'b00, clk_rec_div, clk_rec_div64, clk_tx_div, clk_pll, clk_rx, clk_tx};  
reg [2:0] mon_clock_sel = 3'b000;
wire mon_clock_rate_fresh;
wire [15:0] mon_clock_rate;
eth_f_fmon8 fm0 (
    .clk        (csr_clk),
    .din        (mon_clocks),
    .din_sel    (mon_clock_sel),
    .dout       (mon_clock_rate),
    .dout_fresh (mon_clock_rate_fresh)
);
defparam fm0 .SIM_HURRY   = SIM_HURRY;
defparam fm0 .SIM_EMULATE = SIM_EMULATE;

always @(posedge csr_clk) begin
    if (mon_clock_rate_fresh) begin
        if (mon_clock_sel == 3'b101)
            mon_clock_sel   <=  3'b000;
        else
            mon_clock_sel   <=  mon_clock_sel + 1'b1;
    end
end

reg [31:0]  clk_unused_khz;

//moving the 5th input to 0th mapping of the mux as the previous value is latched for the new mux value inside eth_f_fmon8
always @(posedge csr_clk) begin
    if (mon_clock_rate_fresh) begin
        case (mon_clock_sel)
            3'b000:  clk_rec_div_khz          <=  {16'b0, mon_clock_rate};
            3'b001:  clk_tx_khz          <=  {16'b0, mon_clock_rate};
            3'b010:  clk_rx_khz         <=  {16'b0, mon_clock_rate};
            3'b011:  clk_pll_khz      <=  {16'b0, mon_clock_rate};
            3'b100:  clk_tx_div_khz     <=  {16'b0, mon_clock_rate};
            3'b101:  clk_rec_div64_khz   <=  {16'b0, mon_clock_rate};
            default: clk_unused_khz      <=  {16'b0, mon_clock_rate};
        endcase
    end
end

endmodule // eth_f_clock_mon
