// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

module eth_f_stats_txpp50s (
    input   logic               i_reset,
    input   logic               i_clk,
    input   logic               i_valid_in,
    input   logic               i_valid_out,
    input   logic               i_sop,
    input   logic               i_eop,
    input   logic   [0:1]       i_inframe,
    output  logic   [31:0]      o_starts_in,
    output  logic   [31:0]      o_starts_out,
    output  logic   [31:0]      o_ends_in,
    output  logic   [31:0]      o_ends_out
);

    genvar i;

    logic           start_in;
    logic           end_in;
    logic           starts_out;
    logic           ends_out;
    logic           if_last;

    logic   [0:1]   sop;
    logic   [0:1]   eop;

    assign sop = i_inframe & ~{if_last, i_inframe[0]};
    assign eop = {if_last, i_inframe[0]} & ~i_inframe;

    always_ff @(posedge i_clk) begin
        if (i_reset) begin
            o_starts_out    <= 32'd0;
            o_ends_out      <= 32'd0;
            o_starts_in     <= 32'd0;
            o_ends_in       <= 32'd0;
        end else begin
            o_starts_out    <= 32'(o_starts_out + starts_out);
            o_ends_out      <= 32'(o_ends_out   + ends_out);
            o_starts_in     <= 32'(o_starts_in  + start_in);
            o_ends_in       <= 32'(o_ends_in    + end_in);
        end
    end

    always_ff @(posedge i_clk) begin
        if (i_valid_out) begin
            case (eop)
                2'b01   : ends_out <= 1'd1;
                2'b10   : ends_out <= 1'd1;
                default : ends_out <= 1'd0;
            endcase
        end else begin
            ends_out <= 1'd0;
        end
    end

    always_ff @(posedge i_clk) begin
        if (i_valid_out) begin
            case (sop)
                2'b01   : starts_out <= 1'd1;
                2'b10   : starts_out <= 1'd1;
                default : starts_out <= 1'd0;
            endcase
        end else begin
            starts_out <= 1'd0;
        end
    end

    always_ff @(posedge i_clk) begin
        if (i_reset) begin
            if_last <= 1'b0;
        end else begin
            if (i_valid_out) begin
                if_last <= i_inframe[1];
            end else begin
                if_last <= if_last;
            end
        end
    end

    always_ff @(posedge i_clk) begin
        start_in  <= i_sop && i_valid_in;
        end_in  <= i_eop && i_valid_in;
    end
endmodule