// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//------------------------------------------------------------------------------
// TX AVST to SEGMENT interface bit re-ordering
//------------------------------------------------------------------------------

`timescale 1ps/1ps

module eth_f_tx_avst_reverse #(
       parameter AIB_LANES         = 1
)(
input  [AIB_LANES*64-1:0]  i_tx_data_avst,
input  [AIB_LANES-1:0]     i_tx_inframe_avst,
input  [AIB_LANES*3-1:0]   i_tx_empty_avst,
input  [AIB_LANES-1:0]     i_tx_error_avst,
input  [AIB_LANES-1:0]     i_tx_skip_crc_avst,
output [AIB_LANES*64-1:0]  o_tx_data,
output [AIB_LANES-1:0]     o_tx_inframe,
output [AIB_LANES*3-1:0]   o_tx_empty,
output [AIB_LANES-1:0]     o_tx_skip_crc,
output [AIB_LANES-1:0]     o_tx_error
);

       eth_f_reverse_bytes #(
            .NUM_BYTES      (AIB_LANES*8)
       ) reverse_data_tx (
            .din(i_tx_data_avst),
            .dout(o_tx_data)
       );
       eth_f_reverse_3bits #(
            .NUM_SYMBOL     (AIB_LANES)
       ) reverse_empty_tx (
            .din(i_tx_empty_avst),
            .dout(o_tx_empty)
       );
       eth_f_reverse_bits #(
            .WIDTH      (AIB_LANES)
       ) reverse_inframe_tx (
            .din(i_tx_inframe_avst),
            .dout(o_tx_inframe)
       );
       eth_f_reverse_bits #(
            .WIDTH      (AIB_LANES)
       ) reverse_error_tx (
            .din(i_tx_error_avst),
            .dout(o_tx_error)
       );
       eth_f_reverse_bits #(
            .WIDTH      (AIB_LANES)
       ) reverse_skip_crc_tx (
            .din(i_tx_skip_crc_avst),
            .dout(o_tx_skip_crc)
       );

endmodule
