// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


/*************************************************************************************************************
eth_f_reset_controller.sv
 
Function :  
combines hard resets and soft resets
 
// All resets in this module are active high
**************************************************************************************************************/
// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

module eth_f_reset_logic
#(	
	parameter NUM_CHANNELS = 1,
	parameter SIM_EMULATE = 1
)
(
    // User hard Reset signals
    input wire 			   i_rst, // User reset (~i_rst_n) Resets Transcivers, EHIP
    input wire 			   i_tx_rst, // User reset (~i_tx_rst_n) Resets EHIP TX
    input wire 			   i_rx_rst, // User reset (~i_rx_rst_n) Resets EHIP RX
    
    // Soft CSR reset signals
    input wire 			   i_soft_tx_rst, 
    input wire 			   i_soft_rx_rst,
    input wire 			   i_soft_sys_rst, 

    // Output
    output wire        o_tx_dp_rst, // Reset TX datapath 
    output wire        o_rx_dp_rst // Reset RX datapath    
     
);
   
assign o_tx_dp_rst    = i_rst | i_tx_rst | i_soft_sys_rst | i_soft_tx_rst;
assign o_rx_dp_rst    = i_rst | i_rx_rst | i_soft_sys_rst | i_soft_rx_rst;

endmodule
