// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps

module eth_f_avst_crc_da_tx_cal #(
    parameter WORDS                 = 8,
    parameter EMPTY_WIDTH           = 6
)
  (
   input  logic                     clk,
   input  logic                     rst,
   input  logic                     ena,
   input  logic [WORDS*64-1:0]      i_data,
   input  logic [EMPTY_WIDTH-1:0]   i_empty,
   input  logic                     i_sop,
   input  logic                     i_eop,
   input  logic                     i_vld,
   input  logic                     i_ready,
   input  logic                     i_skip_crc,
   input  logic                     i_error,
   input  logic [48:0]              i_cfg_rom_dest_addr,

   output logic [WORDS*64-1:0]      o_data,
   output logic [EMPTY_WIDTH-1:0]   o_empty,
   output logic                     o_sop,
   output logic                     o_eop,
   output logic                     o_vld,
   output logic                     o_ready,
   output logic                     o_skip_crc,
   output logic                     o_error
   );

    localparam BYTE_CNT = WORDS*8;
    localparam BIT_CNT  = WORDS*64;
    localparam CRC_LEN  = 6'd32;

    logic [BIT_CNT-1:0] [CRC_LEN-1:0] cal_crc32_poly;
    logic [CRC_LEN-1:0] crc32_poly_reg, cal_crc32_poly_reg, crc32, n_crc32;
    logic [BIT_CNT-1:0] crc_i_data, data_r, data_t, data_r1, o_data_n, o_data_t, data_pad_out, data_pad_zero, data_n_pad_zero;
    logic [EMPTY_WIDTH*8-1:0] empty_shift, empty_shift_tx, empty_shift_n;
    logic [EMPTY_WIDTH-1:0] empty_r1,n_empty_t,n_empty, o_empty_t, o_empty_n;
    logic sop_r1, sop_r2, eop_r1, eop_r2, eop_t1, eop_t2, n_vld_sync, n_vld_sync_r1, n_vld, n_vld_r, o_eop_p, 
          emp_new_cycle, emp_new_cycle_sync, ready_en, o_skip_crc_t1, o_skip_crc_t2, o_error_t1, o_error_t2;
    logic [WORDS-1:0][63:0] data_vld_i;
    logic [63:0] data_xor_f_reg, data_xor_t;
    logic [15:0] le_incr;

    assign sop_t1       = sop_r1 & i_ready;
    assign o_sop        = (ena)? sop_r2 & i_ready : i_sop;
    assign eop_t1       = eop_r1 & i_ready;
    assign eop_t2       = eop_r2 & i_ready;

    assign o_eop        = (ena)? (n_vld_sync? 1'b0 : (n_vld_sync_r1? i_ready : eop_t2)) : i_eop;

    assign o_ready      = (ena)? emp_new_cycle_sync & i_ready : i_ready;
    assign o_vld        = (ena)? (!i_sop & i_vld) | ((o_sop | o_eop | n_vld_sync) & i_ready) : i_vld;

    assign o_empty      = (ena)? o_empty_t : i_empty;
    assign o_data       = (ena)? o_data_t : data_t;
    assign o_skip_crc   = (ena)? o_skip_crc_t2 :i_skip_crc;
    assign o_error      = (ena)? o_error_t2 : i_error;


    generate
    if (WORDS == 1) begin
    always_comb begin
        le_incr = '0;
        data_t  =  (i_sop & i_cfg_rom_dest_addr[48])? {i_cfg_rom_dest_addr[47:0],i_data[(BIT_CNT-49):0]} : i_data;;
        if (sop_t1) begin
            le_incr = (ena)? i_data[(BIT_CNT-33)-:16] + 16'd4 : i_data[(BIT_CNT-33)-:16];
            data_t  = {i_data[(BIT_CNT-1)-:32],le_incr,i_data[(BIT_CNT-49):0]};
        end
    end
    end
    else begin
    always_comb begin
        le_incr = '0;
        data_t  = i_data;
        if (i_sop) begin
            le_incr = (ena)? i_data[(BIT_CNT-97)-:16] + 16'd4 : i_data[(BIT_CNT-97)-:16];
            data_t  = i_cfg_rom_dest_addr[48]? {i_cfg_rom_dest_addr[47:0],i_data[(BIT_CNT-49)-:48],le_incr,i_data[(BIT_CNT-113):0]}:{i_data[(BIT_CNT-1)-:96],le_incr,i_data[(BIT_CNT-113):0]};
        end
    end
    end
    endgenerate

    assign emp_new_cycle   = (i_eop && i_empty<3'd4)? 1'b0:1'b1;

    always_ff @(posedge clk) begin
        if (rst) emp_new_cycle_sync  <= '0;
        else if (i_ready)  emp_new_cycle_sync  <= emp_new_cycle;
    end

    

    always_ff @(posedge clk) begin
        if (rst) begin
            sop_r1          <= '0;
            sop_r2          <= '0;
            eop_r1          <= '0;
            eop_r2          <= '0;
            empty_r1        <= '0;
            data_r1         <= '0;
            o_skip_crc_t1   <= '0;
            o_skip_crc_t2   <= '0;
            o_error_t1      <= '0;
            o_error_t2      <= '0;
            n_crc32         <= '0;
            n_vld_sync_r1   <= '0;
        end
        else if (i_ready) begin
            sop_r1          <= i_sop;
            sop_r2          <= sop_t1;
            eop_r1          <= i_eop;
            eop_r2          <= eop_t1;
            empty_r1        <= i_empty;
            data_r1         <= data_t;
            o_skip_crc_t1   <= i_skip_crc;
            o_skip_crc_t2   <= o_skip_crc_t1;
            o_error_t1      <= i_error;
            o_error_t2      <= o_error_t1;
            n_crc32         <= crc32;
            n_vld_sync_r1   <= n_vld_sync;
        end
    end

    
    // Appending CRC to o_data
    generate
    if (WORDS == 1) begin : CRC_Insertion_10G_25G
    always_ff @(posedge clk) begin
        if (rst) begin
            n_vld_sync      <= '0;
            n_empty         <= '0;
            o_data_t        <= '0;
            o_empty_t       <= '0;
        end
        else if (i_ready) begin
            o_data_t        <= data_r1;
            o_empty_t       <= empty_r1;
            if (eop_r1) begin
                if (~empty_r1[EMPTY_WIDTH-1]) begin
                    n_empty             <= empty_r1;
                    n_vld_sync          <= 1'b1;
                    o_empty_t           <= '0;
                    case(empty_r1) 
                        default: o_data_t    <= data_r1;
                        3'd1: o_data_t    <= {data_r1[63:8],crc32[31:24]};
                        3'd2: o_data_t    <= {data_r1[63:16],crc32[31:16]};
                        3'd3: o_data_t    <= {data_r1[63:24],crc32[31:8]};
                    endcase
                end else begin
                    case(empty_r1)
                        default:begin
                            o_data_t    <= {data_r1[63:32],crc32};
                            o_empty_t   <= '0;
                        end
                        3'd5:begin
                            o_data_t    <= {data_r1[63:40],crc32,8'b0};
                            o_empty_t   <= 3'd1;
                        end
                        3'd6:begin
                            o_data_t    <= {data_r1[63:48],crc32,16'b0};
                            o_empty_t   <= 3'd2;
                        end
                        3'd7:begin
                            o_data_t    <= {data_r1[63:56],crc32,24'b0};
                            o_empty_t   <= 3'd3;
                        end
                    endcase
                end
            end else if (n_vld_sync) begin
                n_vld_sync  <= 1'b0;
                case (n_empty)
                    default:   begin 
                        o_data_t      <= {n_crc32,32'd0};
                        o_empty_t     <= 3'd4;
                    end
                    3'd1:begin
                        o_data_t      <= {n_crc32[23:0],40'd0};
                        o_empty_t     <= 3'd5;
                    end
                    3'd2:begin
                        o_data_t      <= {n_crc32[15:0],48'd0};
                        o_empty_t     <= 3'd6;
                    end
                    3'd3:begin
                        o_data_t      <= {n_crc32[7:0],56'd0};
                        o_empty_t     <= 3'd7;
                    end
                endcase
            end
        end
    end
    end
    else if(WORDS == 2) begin : CRC_Insertion_40G_50G
    always_ff @(posedge clk) begin
        if (rst) begin
            n_vld_sync      <= '0;
            n_empty         <= '0;
            o_data_t        <= '0;
            o_empty_t       <= '0;
        end
        else if (i_ready) begin
            o_data_t        <= data_r1;
            o_empty_t       <= empty_r1;
            if (eop_r1) begin
                if (~(|empty_r1[EMPTY_WIDTH-1:EMPTY_WIDTH-2])) begin
                    n_empty             <= empty_r1;
                    n_vld_sync          <= 1'b1;
                    o_empty_t           <= '0;
                    case(empty_r1) 
                        default: o_data_t    <= data_r1;
                        4'd1: o_data_t    <= {data_r1[127:8],crc32[31:24]};
                        4'd2: o_data_t    <= {data_r1[127:16],crc32[31:16]};
                        4'd3: o_data_t    <= {data_r1[127:24],crc32[31:8]};
                    endcase
                end else begin
                    case(empty_r1)
                        default:begin
                            o_data_t    <= {data_r1[127:32],crc32};
                            o_empty_t   <= '0;
                        end
                        4'd5:begin
                            o_data_t    <= {data_r1[127:40],crc32,8'b0};
                            o_empty_t   <= 4'd1;
                        end
                        4'd6:begin
                            o_data_t    <= {data_r1[127:48],crc32,16'b0};
                            o_empty_t   <= 4'd2;
                        end
                        4'd7:begin
                            o_data_t    <= {data_r1[127:56],crc32,24'b0};
                            o_empty_t   <= 4'd3;
                        end
                        4'd8:begin
                            o_data_t    <= {data_r1[127:64],crc32,32'b0};
                            o_empty_t   <= 4'd4;
                        end
                        4'd9:begin
                            o_data_t    <= {data_r1[127:72],crc32,40'b0};
                            o_empty_t   <= 4'd5;
                        end
                        4'd10:begin
                            o_data_t    <= {data_r1[127:80],crc32,48'b0};
                            o_empty_t   <= 4'd6;
                        end
                        4'd11:begin
                            o_data_t    <= {data_r1[127:88],crc32,56'b0};
                            o_empty_t   <= 4'd7;
                        end
                        4'd12:begin
                            o_data_t    <= {data_r1[127:96],crc32,64'b0};
                            o_empty_t   <= 4'd8;
                        end
                        4'd13:begin
                            o_data_t    <= {data_r1[127:104],crc32,72'b0};
                            o_empty_t   <= 4'd9;
                        end
                        4'd14:begin
                            o_data_t    <= {data_r1[127:112],crc32,80'b0};
                            o_empty_t   <= 4'd10;
                        end
                        4'd15:begin
                            o_data_t    <= {data_r1[127:120],crc32,88'b0};
                            o_empty_t   <= 4'd11;
                        end
                    endcase
                end
            end else if (n_vld_sync) begin
                n_vld_sync  <= 1'b0;
                case (n_empty)
                    default:   begin 
                        o_data_t      <= {n_crc32,96'd0};
                        o_empty_t     <= 4'd12;
                    end
                    4'd1:begin
                        o_data_t      <= {n_crc32[23:0],104'd0};
                        o_empty_t     <= 4'd13;
                    end
                    4'd2:begin
                        o_data_t      <= {n_crc32[15:0],112'd0};
                        o_empty_t     <= 4'd14;
                    end
                    4'd3:begin
                        o_data_t      <= {n_crc32[7:0],120'd0};
                        o_empty_t     <= 4'd15;
                    end
                endcase
            end
        end
    end
    end
    else begin : CRC_Insertion_100G
    always_ff @(posedge clk) begin
        if (rst) begin
            n_vld_sync      <= '0;
            n_empty         <= '0;
            o_data_t        <= '0;
            o_empty_t       <= '0;
        end
        else if (i_ready) begin
            o_data_t        <= data_r1;
            o_empty_t       <= empty_r1;
            if (eop_r1) begin
                if (~(|empty_r1[EMPTY_WIDTH-1:EMPTY_WIDTH-4])) begin
                    n_empty             <= empty_r1;
                    n_vld_sync          <= 1'b1;
                    o_empty_t           <= '0;
                    case(empty_r1) 
                        default: o_data_t    <= data_r1;
                        6'd1: o_data_t    <= {data_r1[511:8],crc32[31:24]};
                        6'd2: o_data_t    <= {data_r1[511:16],crc32[31:16]};
                        6'd3: o_data_t    <= {data_r1[511:24],crc32[31:8]};
                    endcase
                end else begin
                    case(empty_r1)
                        default:begin
                            o_data_t    <= {data_r1[511:32],crc32};
                            o_empty_t   <= '0;
                        end
                        6'd5:begin
                            o_data_t    <= {data_r1[511:40],crc32,8'b0};
                            o_empty_t   <= 6'd1;
                        end
                        6'd6:begin
                            o_data_t    <= {data_r1[511:48],crc32,16'b0};
                            o_empty_t   <= 6'd2;
                        end
                        6'd7:begin
                            o_data_t    <= {data_r1[511:56],crc32,24'b0};
                            o_empty_t   <= 6'd3;
                        end
                        6'd8:begin
                            o_data_t    <= {data_r1[511:64],crc32,32'b0};
                            o_empty_t   <= 6'd4;
                        end
                        6'd9:begin
                            o_data_t    <= {data_r1[511:72],crc32,40'b0};
                            o_empty_t   <= 6'd5;
                        end
                        6'd10:begin
                            o_data_t    <= {data_r1[511:80],crc32,48'b0};
                            o_empty_t   <= 6'd6;
                        end
                        6'd11:begin
                            o_data_t    <= {data_r1[511:88],crc32,56'b0};
                            o_empty_t   <= 6'd7;
                        end
                        6'd12:begin
                            o_data_t    <= {data_r1[511:96],crc32,64'b0};
                            o_empty_t   <= 6'd8;
                        end
                        6'd13:begin
                            o_data_t    <= {data_r1[511:104],crc32,72'b0};
                            o_empty_t   <= 6'd9;
                        end
                        6'd14:begin
                            o_data_t    <= {data_r1[511:112],crc32,80'b0};
                            o_empty_t   <= 6'd10;
                        end
                        6'd15:begin
                            o_data_t    <= {data_r1[511:120],crc32,88'b0};
                            o_empty_t   <= 6'd11;
                        end
                        6'd16:begin
                            o_data_t    <= {data_r1[511:128],crc32,96'b0};
                            o_empty_t   <= 6'd12;
                        end
                        6'd17:begin
                            o_data_t    <= {data_r1[511:136],crc32,104'b0};
                            o_empty_t   <= 6'd13;
                        end
                        6'd18:begin
                            o_data_t    <= {data_r1[511:144],crc32,112'b0};
                            o_empty_t   <= 6'd14;
                        end
                        6'd19:begin
                            o_data_t    <= {data_r1[511:152],crc32,120'b0};
                            o_empty_t   <= 6'd15;
                        end
                        6'd20:begin
                            o_data_t    <= {data_r1[511:160],crc32,128'b0};
                            o_empty_t   <= 6'd16;
                        end
                        6'd21:begin
                            o_data_t    <= {data_r1[511:168],crc32,136'b0};
                            o_empty_t   <= 6'd17;
                        end
                        6'd22:begin
                            o_data_t    <= {data_r1[511:176],crc32,144'b0};
                            o_empty_t   <= 6'd18;
                        end
                        6'd23:begin
                            o_data_t    <= {data_r1[511:184],crc32,152'b0};
                            o_empty_t   <= 6'd19;
                        end
                        6'd24:begin
                            o_data_t    <= {data_r1[511:192],crc32,160'b0};
                            o_empty_t   <= 6'd20;
                        end
                        6'd25:begin
                            o_data_t    <= {data_r1[511:200],crc32,168'b0};
                            o_empty_t   <= 6'd21;
                        end
                        6'd26:begin
                            o_data_t    <= {data_r1[511:208],crc32,176'b0};
                            o_empty_t   <= 6'd22;
                        end
                        6'd27:begin
                            o_data_t    <= {data_r1[511:216],crc32,184'b0};
                            o_empty_t   <= 6'd23;
                        end
                        6'd28:begin
                            o_data_t    <= {data_r1[511:224],crc32,192'b0};
                            o_empty_t   <= 6'd24;
                        end
                        6'd29:begin
                            o_data_t    <= {data_r1[511:232],crc32,200'b0};
                            o_empty_t   <= 6'd25;
                        end
                        6'd30:begin
                            o_data_t    <= {data_r1[511:240],crc32,208'b0};
                            o_empty_t   <= 6'd26;
                        end
                        6'd31:begin
                            o_data_t    <= {data_r1[511:248],crc32,216'b0};
                            o_empty_t   <= 6'd27;
                        end
                        6'd32:begin
                            o_data_t    <= {data_r1[511:256],crc32,224'b0};
                            o_empty_t   <= 6'd28;
                        end
                        6'd33:begin
                            o_data_t    <= {data_r1[511:264],crc32,232'b0};
                            o_empty_t   <= 6'd29;
                        end
                        6'd34:begin
                            o_data_t    <= {data_r1[511:272],crc32,240'b0};
                            o_empty_t   <= 6'd30;
                        end
                        6'd35:begin
                            o_data_t    <= {data_r1[511:280],crc32,248'b0};
                            o_empty_t   <= 6'd31;
                        end
                        6'd36:begin
                            o_data_t    <= {data_r1[511:288],crc32,256'b0};
                            o_empty_t   <= 6'd32;
                        end
                        6'd37:begin
                            o_data_t    <= {data_r1[511:296],crc32,264'b0};
                            o_empty_t   <= 6'd33;
                        end
                        6'd38:begin
                            o_data_t    <= {data_r1[511:304],crc32,272'b0};
                            o_empty_t   <= 6'd34;
                        end
                        6'd39:begin
                            o_data_t    <= {data_r1[511:312],crc32,280'b0};
                            o_empty_t   <= 6'd35;
                        end
                        6'd40:begin
                            o_data_t    <= {data_r1[511:320],crc32,288'b0};
                            o_empty_t   <= 6'd36;
                        end
                        6'd41:begin
                            o_data_t    <= {data_r1[511:328],crc32,296'b0};
                            o_empty_t   <= 6'd37;
                        end
                        6'd42:begin
                            o_data_t    <= {data_r1[511:336],crc32,304'b0};
                            o_empty_t   <= 6'd38;
                        end
                        6'd43:begin
                            o_data_t    <= {data_r1[511:344],crc32,312'b0};
                            o_empty_t   <= 6'd39;
                        end
                        6'd44:begin
                            o_data_t    <= {data_r1[511:352],crc32,320'b0};
                            o_empty_t   <= 6'd40;
                        end
                        6'd45:begin
                            o_data_t    <= {data_r1[511:360],crc32,328'b0};
                            o_empty_t   <= 6'd41;
                        end
                        6'd46:begin
                            o_data_t    <= {data_r1[511:368],crc32,336'b0};
                            o_empty_t   <= 6'd42;
                        end
                        6'd47:begin
                            o_data_t    <= {data_r1[511:376],crc32,344'b0};
                            o_empty_t   <= 6'd43;
                        end
                        6'd48:begin
                            o_data_t    <= {data_r1[511:384],crc32,352'b0};
                            o_empty_t   <= 6'd44;
                        end
                        6'd49:begin
                            o_data_t    <= {data_r1[511:392],crc32,360'b0};
                            o_empty_t   <= 6'd45;
                        end
                        6'd50:begin
                            o_data_t    <= {data_r1[511:400],crc32,368'b0};
                            o_empty_t   <= 6'd46;
                        end
                        6'd51:begin
                            o_data_t    <= {data_r1[511:408],crc32,376'b0};
                            o_empty_t   <= 6'd47;
                        end
                        6'd52:begin
                            o_data_t    <= {data_r1[511:416],crc32,384'b0};
                            o_empty_t   <= 6'd48;
                        end
                        6'd53:begin
                            o_data_t    <= {data_r1[511:424],crc32,392'b0};
                            o_empty_t   <= 6'd49;
                        end
                        6'd54:begin
                            o_data_t    <= {data_r1[511:432],crc32,400'b0};
                            o_empty_t   <= 6'd50;
                        end
                        6'd55:begin
                            o_data_t    <= {data_r1[511:440],crc32,408'b0};
                            o_empty_t   <= 6'd51;
                        end
                        6'd56:begin
                            o_data_t    <= {data_r1[511:448],crc32,416'b0};
                            o_empty_t   <= 6'd52;
                        end
                        6'd57:begin
                            o_data_t    <= {data_r1[511:456],crc32,424'b0};
                            o_empty_t   <= 6'd53;
                        end
                        6'd58:begin
                            o_data_t    <= {data_r1[511:464],crc32,432'b0};
                            o_empty_t   <= 6'd54;
                        end
                        6'd59:begin
                            o_data_t    <= {data_r1[511:472],crc32,440'b0};
                            o_empty_t   <= 6'd55;
                        end
                        6'd60:begin
                            o_data_t    <= {data_r1[511:480],crc32,448'b0};
                            o_empty_t   <= 6'd56;
                        end
                        6'd61:begin
                            o_data_t    <= {data_r1[511:488],crc32,456'b0};
                            o_empty_t   <= 6'd57;
                        end
                        6'd62:begin
                            o_data_t    <= {data_r1[511:496],crc32,464'b0};
                            o_empty_t   <= 6'd58;
                        end
                        6'd63:begin
                            o_data_t    <= {data_r1[511:504],crc32,472'b0};
                            o_empty_t   <= 6'd59;
                        end
                    endcase
                end
            end else if (n_vld_sync) begin //new cycle if empty < 4
                n_vld_sync  <= 1'b0;
                case (n_empty)
                    default:begin 
                        o_data_t      <= {n_crc32,480'd0};
                        o_empty_t     <= 6'd60;
                    end
                    6'd1:begin
                        o_data_t      <= {n_crc32[23:0],488'd0};
                        o_empty_t     <= 6'd61;
                    end
                    6'd2:begin
                        o_data_t      <= {n_crc32[15:0],496'd0};
                        o_empty_t     <= 6'd62;
                    end
                    6'd3:begin
                        o_data_t      <= {n_crc32[7:0],504'd0};
                        o_empty_t     <= 6'd63;
                    end
                endcase
            end
        end
    end
    end
    
    endgenerate
                
                
    // storing data as 64b MDA for xoring 
    genvar slice_d;
    generate
    for (slice_d=0; slice_d<WORDS; slice_d++) begin : SLICE_ITERATE_64b
    always_comb begin
        data_vld_i[slice_d]   = '0;
        if (i_ready & i_vld)
            data_vld_i[slice_d] = data_t[((slice_d+1)*64)-1:slice_d*64];
    end
    end
    endgenerate

    // xoring array of 64b data
    always_comb begin
        data_xor_t  = '0;
        if (i_ready) 
            for(int i=0; i<WORDS; i++) 
                data_xor_t  = data_xor_t ^ data_vld_i[i];
    end

// initalizing xored memory when sop comes otherwise xor with previous and store
    always_ff @(posedge clk) begin
        if (rst) begin
            data_xor_f_reg  <= '0;
        end else if (i_ready) begin
            data_xor_f_reg  <= (i_sop & i_ready)? data_xor_t : data_xor_t ^ data_xor_f_reg;
        end
    end

// calculate crc based on CRC-32/MPEG-2 - https://crccalc.com/
    always @(*) begin
        crc32[0] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[10] ^ data_xor_f_reg[12] ^ data_xor_f_reg[16] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[34] ^ data_xor_f_reg[37] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32[1] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[24] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[53] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32[2] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[44] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59];
        crc32[3] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[45] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60];
        crc32[4] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[6] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[15] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[63];
        crc32[5] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[13] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[24] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32[6] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[14] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[25] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32[7] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[5] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[10] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[34] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60];
        crc32[8] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[8] ^ data_xor_f_reg[10] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[17] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[63];
        crc32[9] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[18] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[55] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61];
        crc32[10] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[5] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[19] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[50] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32[11] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[20] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59];
        crc32[12] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[21] ^ data_xor_f_reg[24] ^ data_xor_f_reg[27] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32[13] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[22] ^ data_xor_f_reg[25] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32[14] = 0 ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[23] ^ data_xor_f_reg[26] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32[15] = 1 ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[18] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[24] ^ data_xor_f_reg[27] ^ data_xor_f_reg[30] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32[16] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[8] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[17] ^ data_xor_f_reg[19] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[51] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57];
        crc32[17] = 0 ^ data_xor_f_reg[1] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[18] ^ data_xor_f_reg[20] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[38] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[52] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58];
        crc32[18] = 1 ^ data_xor_f_reg[2] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[19] ^ data_xor_f_reg[21] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[34] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[46] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[53] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59];
        crc32[19] = 0 ^ data_xor_f_reg[3] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[20] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[35] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[54] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60];
        crc32[20] = 0 ^ data_xor_f_reg[4] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[21] ^ data_xor_f_reg[23] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[30] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[36] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61];
        crc32[21] = 0 ^ data_xor_f_reg[5] ^ data_xor_f_reg[9] ^ data_xor_f_reg[10] ^ data_xor_f_reg[13] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[49] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[56] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32[22] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32[23] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[42] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32[24] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[30] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[43] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32[25] = 0 ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[44] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32[26] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[6] ^ data_xor_f_reg[10] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[44] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32[27] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[7] ^ data_xor_f_reg[11] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[53] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32[28] = 0 ^ data_xor_f_reg[2] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[8] ^ data_xor_f_reg[12] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[30] ^ data_xor_f_reg[33] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32[29] = 1 ^ data_xor_f_reg[3] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32[30] = 1 ^ data_xor_f_reg[4] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[48] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32[31] = 0 ^ data_xor_f_reg[5] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[15] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
    end

    
endmodule