// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps

module eth_f_avst_crc_rx_chk #(
    parameter WORDS                 = 8,
    parameter EMPTY_WIDTH           = 8
)
  (
   input  logic                     clk,
   input  logic                     rst,
   input  logic                     ena,
   input  logic [WORDS*64-1:0]      i_data,
   input  logic [EMPTY_WIDTH-1:0]   i_empty,
   input  logic                     i_sop,
   input  logic                     i_eop,
   input  logic                     i_vld,

   output logic [7:0] stat_err_cnt
   );

    localparam BYTE_CNT = WORDS*8;
    localparam BIT_CNT  = WORDS*64;
    localparam CRC_LEN  = 6'd32;

    logic [BIT_CNT-1:0] [CRC_LEN-1:0] cal_crc32_poly;
    logic [CRC_LEN-1:0] crc32_poly_reg, cal_crc32_poly_reg, crc32_out, crc32_out_t, crc_in, crc32_poly_reg_1, crc_in_t, crc_in_l;
    logic [BIT_CNT-1:0] crc_i_data, data_pad_zero, data_i_r1, data_i_r2, crc_in_pad_cur, crc_in_pad_prev;
    logic [EMPTY_WIDTH*8-1:0] empty_shift, empty_shift_rx, empty_shift_prev;

    logic cnt_vld, n_vld, eop_r1, eop_r2, eop_t1, eop_t2, sop_r1, sop_r2, sop_t1, sop_t2;
    logic [7:0] stat_err_cnt_t;
    logic [3:0] clk_cnt;
    logic [63:0] p_crc_data, data_xor_f_reg, data_xor, data_xor_t;
    logic [EMPTY_WIDTH-1:0] empty_i_r1, empty_i_r2;
    logic [WORDS-1:0][63:0] data_vld_i;

    assign eop_t1 = eop_r1 & i_vld;
    assign eop_t2 = eop_r2 & i_vld;
    assign sop_t1 = sop_r1 & i_vld;
    assign sop_t2 = sop_r2 & i_vld;


    always_ff @(posedge clk) begin 
        if (rst) begin
            stat_err_cnt_t  <= '0;
            cnt_vld         <= 1'b0;
            eop_r1          <= '0;
            eop_r2          <= '0;
            sop_r1          <= '0;
            sop_r2          <= '0;
            data_i_r1       <= '0;
            data_i_r2       <= '0;
            p_crc_data      <= '0;
            empty_i_r1      <= '0;
            empty_i_r2      <= '0;
        end
        else if (i_vld) begin
            eop_r1          <= i_eop;
            eop_r2          <= eop_t1;
            sop_r1          <= i_sop;
            sop_r2          <= sop_t1;
            empty_i_r1      <= i_empty;
            empty_i_r2      <= empty_i_r1;
            data_i_r1       <= i_data;
            data_i_r2       <= data_i_r1;
            p_crc_data      <= data_i_r1[63:0];
        end
        cnt_vld             <= (i_vld & eop_t2)? 1'b1 : 1'b0;
        if (cnt_vld & ena)
            stat_err_cnt_t  <= stat_err_cnt_t + (crc_in != crc32_out);
    end


    always_ff @(posedge clk) begin
        if (rst)    clk_cnt <= 4'b0;
        else        clk_cnt <= clk_cnt + 4'b1;
    end

    always_ff @(posedge clk) begin
        if (rst)    stat_err_cnt <= 0;
        else if (&clk_cnt)  stat_err_cnt <= stat_err_cnt_t;
    end

// CRC extraction from data
    generate
    if (WORDS == 1) begin : CRC_Extraction_10G_25G
    always_ff @(posedge clk) begin
        if (rst)
            crc_in_t <= '0;
        else if (i_vld & eop_t1)   
            case (empty_i_r1) 
                3'd0 : crc_in_t <=  data_i_r1[31-:32];
                3'd1 : crc_in_t <=  data_i_r1[39-:32];
                3'd2 : crc_in_t <=  data_i_r1[47-:32];
                3'd3 : crc_in_t <=  data_i_r1[55-:32];
                3'd4 : crc_in_t <=  data_i_r1[63-:32];
                3'd5 : crc_in_t <=  {p_crc_data[7:0],data_i_r1[63-:24]};
                3'd6 : crc_in_t <=  {p_crc_data[15:0],data_i_r1[63-:16]};
                3'd7 : crc_in_t <=  {p_crc_data[23:0],data_i_r1[63-:8]};
            endcase
    end
    end
    else if (WORDS == 2) begin : CRC_Extraction_40G_50G
    always_ff @(posedge clk) begin
        if (rst)
            crc_in_t <= '0;
        else if (i_vld & eop_t1)  
            case (empty_i_r1) 
                4'd0 : crc_in_t <=  data_i_r1[31-:32];
                4'd1 : crc_in_t <=  data_i_r1[39-:32];
                4'd2 : crc_in_t <=  data_i_r1[47-:32];
                4'd3 : crc_in_t <=  data_i_r1[55-:32];
                4'd4 : crc_in_t <=  data_i_r1[63-:32];
                4'd5 : crc_in_t <=  data_i_r1[71-:32];
                4'd6 : crc_in_t <=  data_i_r1[79-:32];
                4'd7 : crc_in_t <=  data_i_r1[87-:32];
                4'd8 : crc_in_t <=  data_i_r1[95-:32];
                4'd9 : crc_in_t <=  data_i_r1[103-:32];
                4'd10 : crc_in_t <=  data_i_r1[111-:32];
                4'd11 : crc_in_t <=  data_i_r1[119-:32];
                4'd12 : crc_in_t <=  data_i_r1[127-:32];
                4'd13 : crc_in_t <=  {p_crc_data[7:0],data_i_r1[127-:24]};
                4'd14 : crc_in_t <=  {p_crc_data[15:0],data_i_r1[127-:16]};
                4'd15 : crc_in_t <=  {p_crc_data[23:0],data_i_r1[127-:8]};
            endcase
    end
    end
    else begin : CRC_Extraction_100G
    always_ff @(posedge clk) begin
        if (rst)
            crc_in_t <= '0;
        else if (i_vld & eop_t1)   
            case (empty_i_r1) 
                6'd0 : crc_in_t <=  data_i_r1[31-:32];
                6'd1 : crc_in_t <=  data_i_r1[39-:32];
                6'd2 : crc_in_t <=  data_i_r1[47-:32];
                6'd3 : crc_in_t <=  data_i_r1[55-:32];
                6'd4 : crc_in_t <=  data_i_r1[63-:32];
                6'd5 : crc_in_t <=  data_i_r1[71-:32];
                6'd6 : crc_in_t <=  data_i_r1[79-:32];
                6'd7 : crc_in_t <=  data_i_r1[87-:32];
                6'd8 : crc_in_t <=  data_i_r1[95-:32];
                6'd9 : crc_in_t <=  data_i_r1[103-:32];
                6'd10 : crc_in_t <=  data_i_r1[111-:32];
                6'd11 : crc_in_t <=  data_i_r1[119-:32];
                6'd12 : crc_in_t <=  data_i_r1[127-:32];
                6'd13 : crc_in_t <=  data_i_r1[135-:32];
                6'd14 : crc_in_t <=  data_i_r1[143-:32];
                6'd15 : crc_in_t <=  data_i_r1[151-:32];
                6'd16 : crc_in_t <=  data_i_r1[159-:32];
                6'd17 : crc_in_t <=  data_i_r1[167-:32];
                6'd18 : crc_in_t <=  data_i_r1[175-:32];
                6'd19 : crc_in_t <=  data_i_r1[183-:32];
                6'd20 : crc_in_t <=  data_i_r1[191-:32];
                6'd21 : crc_in_t <=  data_i_r1[199-:32];
                6'd22 : crc_in_t <=  data_i_r1[207-:32];
                6'd23 : crc_in_t <=  data_i_r1[215-:32];
                6'd24 : crc_in_t <=  data_i_r1[223-:32];
                6'd25 : crc_in_t <=  data_i_r1[231-:32];
                6'd26 : crc_in_t <=  data_i_r1[239-:32];
                6'd27 : crc_in_t <=  data_i_r1[247-:32];
                6'd28 : crc_in_t <=  data_i_r1[255-:32];
                6'd29 : crc_in_t <=  data_i_r1[263-:32];
                6'd30 : crc_in_t <=  data_i_r1[271-:32];
                6'd31 : crc_in_t <=  data_i_r1[279-:32];
                6'd32 : crc_in_t <=  data_i_r1[287-:32];
                6'd33 : crc_in_t <=  data_i_r1[295-:32];
                6'd34 : crc_in_t <=  data_i_r1[303-:32];
                6'd35 : crc_in_t <=  data_i_r1[311-:32];
                6'd36 : crc_in_t <=  data_i_r1[319-:32];
                6'd37 : crc_in_t <=  data_i_r1[327-:32];
                6'd38 : crc_in_t <=  data_i_r1[335-:32];
                6'd39 : crc_in_t <=  data_i_r1[343-:32];
                6'd40 : crc_in_t <=  data_i_r1[351-:32];
                6'd41 : crc_in_t <=  data_i_r1[359-:32];
                6'd42 : crc_in_t <=  data_i_r1[367-:32];
                6'd43 : crc_in_t <=  data_i_r1[375-:32];
                6'd44 : crc_in_t <=  data_i_r1[383-:32];
                6'd45 : crc_in_t <=  data_i_r1[391-:32];
                6'd46 : crc_in_t <=  data_i_r1[399-:32];
                6'd47 : crc_in_t <=  data_i_r1[407-:32];
                6'd48 : crc_in_t <=  data_i_r1[415-:32];
                6'd49 : crc_in_t <=  data_i_r1[423-:32];
                6'd50 : crc_in_t <=  data_i_r1[431-:32];
                6'd51 : crc_in_t <=  data_i_r1[439-:32];
                6'd52 : crc_in_t <=  data_i_r1[447-:32];
                6'd53 : crc_in_t <=  data_i_r1[455-:32];
                6'd54 : crc_in_t <=  data_i_r1[463-:32];
                6'd55 : crc_in_t <=  data_i_r1[471-:32];
                6'd56 : crc_in_t <=  data_i_r1[479-:32];
                6'd57 : crc_in_t <=  data_i_r1[487-:32];
                6'd58 : crc_in_t <=  data_i_r1[495-:32];
                6'd59 : crc_in_t <=  data_i_r1[503-:32];
                6'd60 : crc_in_t <=  data_i_r1[511-:32];
                6'd61 : crc_in_t <=  {p_crc_data[7:0],data_i_r1[511-:24]};
                6'd62 : crc_in_t <=  {p_crc_data[15:0],data_i_r1[511-:16]};
                6'd63 : crc_in_t <=  {p_crc_data[23:0],data_i_r1[511-:8]};
            endcase
    end
    end
    endgenerate

  // storing data as 64b MDA for xoring 
    genvar slice_d;
    generate
    for (slice_d=0; slice_d<WORDS; slice_d++) begin : SLICE_ITERATE_64b
    always_comb begin
        data_vld_i[slice_d]   = '0;
        if (i_vld & !i_eop)
            data_vld_i[slice_d] = i_data[((slice_d+1)*64)-1:slice_d*64];
    end
    end
    endgenerate

 // xoring array of 64b data
    always_comb begin
        data_xor_t  = '0;
        if (i_vld) 
            for(int i=0; i<WORDS; i++) 
                data_xor_t  = data_xor_t ^ data_vld_i[i];
    end
    
// initalizing and removing crc from data when eop comes
    generate
    if (WORDS == 1) begin : Data_Extraction_10G_25G
    always_ff @(posedge clk) begin
        if (rst) data_xor_f_reg <= '0;
        else if (i_vld) begin
            data_xor_f_reg <= (sop_t1)? data_xor : data_xor_f_reg ^ data_xor;
            if (eop_t1) begin
                case (empty_i_r1)
                    3'd0 : data_xor_f_reg <=  data_xor ^ {data_i_r1[63:32],32'b0} ^ data_xor_f_reg;
                    3'd1 : data_xor_f_reg <=  data_xor ^ {data_i_r1[63:40],40'b0} ^ data_xor_f_reg;
                    3'd2 : data_xor_f_reg <=  data_xor ^ {data_i_r1[63:48],48'b0} ^ data_xor_f_reg;
                    3'd3 : data_xor_f_reg <=  data_xor ^ {data_i_r1[63:56],56'b0} ^ data_xor_f_reg;
                    3'd4 : data_xor_f_reg <=  data_xor_f_reg;
                    3'd5 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:8],8'b0}; 
                    3'd6 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:16],16'b0};
                    3'd7 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:24],24'b0};
                endcase
            end
        end
    end
    end else if (WORDS == 2) begin : Data_Extraction_40G_50G
    always_ff @(posedge clk) begin
        if (rst) data_xor_f_reg <= '0;
        else if (i_vld) begin
            data_xor_f_reg <= (sop_t1)? data_xor : data_xor_f_reg ^ data_xor;
            if (eop_t1) begin
                case (empty_i_r1)
                    4'd0 :  data_xor_f_reg <=  data_xor ^ data_i_r1[127:64] ^ {data_i_r1[63:32],32'b0} ^ data_xor_f_reg;
                    4'd1 :  data_xor_f_reg <=  data_xor ^ data_i_r1[127:64] ^ {data_i_r1[63:40],40'b0} ^ data_xor_f_reg;
                    4'd2 :  data_xor_f_reg <=  data_xor ^ data_i_r1[127:64] ^ {data_i_r1[63:48],48'b0} ^ data_xor_f_reg;
                    4'd3 :  data_xor_f_reg <=  data_xor ^ data_i_r1[127:64] ^ {data_i_r1[63:56],56'b0} ^ data_xor_f_reg;
                    4'd4 :  data_xor_f_reg <=  data_xor ^ data_i_r1[127:64] ^ data_xor_f_reg;
                    4'd5 :  data_xor_f_reg <=  data_xor ^ {data_i_r1[127:72],8'b0} ^ data_xor_f_reg;
                    4'd6 :  data_xor_f_reg <=  data_xor ^ {data_i_r1[127:80],16'b0} ^ data_xor_f_reg;
                    4'd7 :  data_xor_f_reg <=  data_xor ^ {data_i_r1[127:88],24'b0} ^ data_xor_f_reg;
                    4'd8 :  data_xor_f_reg <=  data_xor ^ {data_i_r1[127:96],32'b0} ^ data_xor_f_reg;
                    4'd9 :  data_xor_f_reg <=  data_xor ^ {data_i_r1[127:104],40'b0} ^ data_xor_f_reg;
                    4'd10 : data_xor_f_reg <=  data_xor ^ {data_i_r1[127:112],48'b0} ^ data_xor_f_reg;
                    4'd11 : data_xor_f_reg <=  data_xor ^ {data_i_r1[127:120],56'b0} ^ data_xor_f_reg;
                    4'd12 : data_xor_f_reg <=  data_xor_f_reg;
                    4'd13 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:8],8'b0}; 
                    4'd14 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:16],16'b0};
                    4'd15 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:24],24'b0};
                endcase
            end
        end
    end
    end else begin : Data_Extraction_100G 
    always_ff @(posedge clk) begin
        if (rst) data_xor_f_reg <= '0;
        else if (i_vld) begin
            data_xor_f_reg <= (sop_t1)? data_xor : data_xor_f_reg ^ data_xor;
            if (eop_t1) begin
                case (empty_i_r1)
                    6'd0 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ data_i_r1[127:64] ^ {data_i_r1[63:32],32'b0} ^ data_xor_f_reg;
                    6'd1 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ data_i_r1[127:64] ^ {data_i_r1[63:40],40'b0} ^ data_xor_f_reg;
                    6'd2 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ data_i_r1[127:64] ^ {data_i_r1[63:48],48'b0} ^ data_xor_f_reg;
                    6'd3 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ data_i_r1[127:64] ^ {data_i_r1[63:56],56'b0} ^ data_xor_f_reg;
                    6'd4 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ data_i_r1[127:64] ^ data_xor_f_reg;
                    6'd5 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:72],8'b0} ^ data_xor_f_reg;
                    6'd6 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:80],16'b0} ^ data_xor_f_reg;
                    6'd7 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:88],24'b0} ^ data_xor_f_reg;
                    6'd8 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:96],32'b0} ^ data_xor_f_reg;
                    6'd9 :  data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:104],40'b0} ^ data_xor_f_reg;
                    6'd10 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:112],48'b0} ^ data_xor_f_reg;
                    6'd11 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ {data_i_r1[127:120],56'b0} ^ data_xor_f_reg;
                    6'd12 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_i_r1[191:128] ^ data_xor_f_reg;
                    6'd13 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:136],8'b0} ^ data_xor_f_reg;
                    6'd14 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:144],16'b0} ^ data_xor_f_reg;
                    6'd15 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:152],24'b0} ^ data_xor_f_reg;
                    6'd16 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:160],32'b0} ^ data_xor_f_reg;
                    6'd17 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:168],40'b0} ^ data_xor_f_reg;
                    6'd18 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:176],48'b0} ^ data_xor_f_reg;
                    6'd19 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ {data_i_r1[191:184],56'b0} ^ data_xor_f_reg;
                    6'd20 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_i_r1[255:192] ^ data_xor_f_reg;
                    6'd21 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:200],8'b0} ^ data_xor_f_reg;
                    6'd22 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:208],16'b0} ^ data_xor_f_reg;
                    6'd23 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:216],24'b0} ^ data_xor_f_reg;
                    6'd24 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:224],32'b0} ^ data_xor_f_reg;
                    6'd25 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:232],40'b0} ^ data_xor_f_reg;
                    6'd26 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:240],48'b0} ^ data_xor_f_reg;
                    6'd27 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ {data_i_r1[255:248],56'b0} ^ data_xor_f_reg;
                    6'd28 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_i_r1[319:256] ^ data_xor_f_reg;
                    6'd29 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:264],8'b0} ^ data_xor_f_reg;
                    6'd30 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:272],16'b0} ^ data_xor_f_reg;
                    6'd31 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:280],24'b0} ^ data_xor_f_reg;
                    6'd32 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:288],32'b0} ^ data_xor_f_reg;
                    6'd33 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:296],40'b0} ^ data_xor_f_reg;
                    6'd34 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:304],48'b0} ^ data_xor_f_reg;
                    6'd35 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ {data_i_r1[319:312],56'b0} ^ data_xor_f_reg;
                    6'd36 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_i_r1[383:320] ^ data_xor_f_reg;
                    6'd37 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:328],8'b0} ^ data_xor_f_reg;
                    6'd38 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:336],16'b0} ^ data_xor_f_reg;
                    6'd39 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:344],24'b0} ^ data_xor_f_reg;
                    6'd40 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:352],32'b0} ^ data_xor_f_reg;
                    6'd41 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:360],40'b0} ^ data_xor_f_reg;
                    6'd42 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:368],48'b0} ^ data_xor_f_reg;
                    6'd43 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ {data_i_r1[383:376],56'b0} ^ data_xor_f_reg;
                    6'd44 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_i_r1[447:384] ^ data_xor_f_reg;
                    6'd45 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:392],8'b0} ^ data_xor_f_reg;
                    6'd46 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:400],16'b0} ^ data_xor_f_reg;
                    6'd47 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:408],24'b0} ^ data_xor_f_reg;
                    6'd48 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:416],32'b0} ^ data_xor_f_reg;
                    6'd49 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:424],40'b0} ^ data_xor_f_reg;
                    6'd50 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:432],48'b0} ^ data_xor_f_reg;
                    6'd51 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ {data_i_r1[447:440],56'b0} ^ data_xor_f_reg;
                    6'd52 : data_xor_f_reg <=  data_xor ^ data_i_r1[511:448] ^ data_xor_f_reg;
                    6'd53 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:456],8'b0} ^ data_xor_f_reg;
                    6'd54 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:464],16'b0} ^ data_xor_f_reg;
                    6'd55 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:472],24'b0} ^ data_xor_f_reg;
                    6'd56 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:480],32'b0} ^ data_xor_f_reg;
                    6'd57 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:488],40'b0} ^ data_xor_f_reg;
                    6'd58 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:496],48'b0} ^ data_xor_f_reg;
                    6'd59 : data_xor_f_reg <=  data_xor ^ {data_i_r1[511:504],56'b0} ^ data_xor_f_reg;
                    6'd60 : data_xor_f_reg <=  data_xor_f_reg;
                    6'd61 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:8],8'b0}; 
                    6'd62 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:16],16'b0};
                    6'd63 : data_xor_f_reg <=  data_xor_f_reg ^ p_crc_data ^ {p_crc_data[63:24],24'b0};
                endcase
            end
        end
    end
    end
    endgenerate


    always_ff @(posedge clk) begin
        if (rst) begin
            crc32_out   <= '1;
            data_xor    <= '0;
            crc_in      <= '0;
        end else if (i_vld ) begin
            crc32_out   <= crc32_out_t;
            data_xor    <= data_xor_t;
            crc_in      <= crc_in_t;
        end
    end


// calculate crc based on CRC-32/MPEG-2 - https://crccalc.com/
     always @(*) begin
        crc32_out_t[0] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[10] ^ data_xor_f_reg[12] ^ data_xor_f_reg[16] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[34] ^ data_xor_f_reg[37] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32_out_t[1] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[24] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[53] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32_out_t[2] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[44] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59];
        crc32_out_t[3] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[45] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60];
        crc32_out_t[4] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[6] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[15] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[63];
        crc32_out_t[5] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[13] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[24] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32_out_t[6] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[14] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[25] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32_out_t[7] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[5] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[10] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[34] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60];
        crc32_out_t[8] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[8] ^ data_xor_f_reg[10] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[17] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[63];
        crc32_out_t[9] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[18] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[55] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61];
        crc32_out_t[10] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[5] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[19] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[50] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32_out_t[11] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[20] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59];
        crc32_out_t[12] = 1 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[21] ^ data_xor_f_reg[24] ^ data_xor_f_reg[27] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32_out_t[13] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[22] ^ data_xor_f_reg[25] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32_out_t[14] = 0 ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[23] ^ data_xor_f_reg[26] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32_out_t[15] = 1 ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[18] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[24] ^ data_xor_f_reg[27] ^ data_xor_f_reg[30] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32_out_t[16] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[8] ^ data_xor_f_reg[12] ^ data_xor_f_reg[13] ^ data_xor_f_reg[17] ^ data_xor_f_reg[19] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[51] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57];
        crc32_out_t[17] = 0 ^ data_xor_f_reg[1] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[14] ^ data_xor_f_reg[18] ^ data_xor_f_reg[20] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[38] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[52] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58];
        crc32_out_t[18] = 1 ^ data_xor_f_reg[2] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[15] ^ data_xor_f_reg[19] ^ data_xor_f_reg[21] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[32] ^ data_xor_f_reg[34] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[46] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[53] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59];
        crc32_out_t[19] = 0 ^ data_xor_f_reg[3] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[20] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[33] ^ data_xor_f_reg[35] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[54] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60];
        crc32_out_t[20] = 0 ^ data_xor_f_reg[4] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[12] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[21] ^ data_xor_f_reg[23] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[30] ^ data_xor_f_reg[33] ^ data_xor_f_reg[34] ^ data_xor_f_reg[36] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61];
        crc32_out_t[21] = 0 ^ data_xor_f_reg[5] ^ data_xor_f_reg[9] ^ data_xor_f_reg[10] ^ data_xor_f_reg[13] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[37] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[49] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[56] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32_out_t[22] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[12] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32_out_t[23] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[1] ^ data_xor_f_reg[6] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[15] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[42] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
        crc32_out_t[24] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[2] ^ data_xor_f_reg[7] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[16] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[30] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[43] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32_out_t[25] = 0 ^ data_xor_f_reg[2] ^ data_xor_f_reg[3] ^ data_xor_f_reg[8] ^ data_xor_f_reg[11] ^ data_xor_f_reg[15] ^ data_xor_f_reg[17] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[37] ^ data_xor_f_reg[38] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[44] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32_out_t[26] = 0 ^ data_xor_f_reg[0] ^ data_xor_f_reg[3] ^ data_xor_f_reg[4] ^ data_xor_f_reg[6] ^ data_xor_f_reg[10] ^ data_xor_f_reg[18] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[28] ^ data_xor_f_reg[31] ^ data_xor_f_reg[38] ^ data_xor_f_reg[39] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[44] ^ data_xor_f_reg[47] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[52] ^ data_xor_f_reg[54] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62];
        crc32_out_t[27] = 1 ^ data_xor_f_reg[1] ^ data_xor_f_reg[4] ^ data_xor_f_reg[5] ^ data_xor_f_reg[7] ^ data_xor_f_reg[11] ^ data_xor_f_reg[19] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[29] ^ data_xor_f_reg[32] ^ data_xor_f_reg[39] ^ data_xor_f_reg[40] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[48] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[53] ^ data_xor_f_reg[55] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32_out_t[28] = 0 ^ data_xor_f_reg[2] ^ data_xor_f_reg[5] ^ data_xor_f_reg[6] ^ data_xor_f_reg[8] ^ data_xor_f_reg[12] ^ data_xor_f_reg[20] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[30] ^ data_xor_f_reg[33] ^ data_xor_f_reg[40] ^ data_xor_f_reg[41] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[49] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[54] ^ data_xor_f_reg[56] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32_out_t[29] = 1 ^ data_xor_f_reg[3] ^ data_xor_f_reg[6] ^ data_xor_f_reg[7] ^ data_xor_f_reg[9] ^ data_xor_f_reg[13] ^ data_xor_f_reg[21] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[25] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[31] ^ data_xor_f_reg[34] ^ data_xor_f_reg[41] ^ data_xor_f_reg[42] ^ data_xor_f_reg[44] ^ data_xor_f_reg[45] ^ data_xor_f_reg[47] ^ data_xor_f_reg[50] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[55] ^ data_xor_f_reg[57] ^ data_xor_f_reg[58] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62] ^ data_xor_f_reg[63];
        crc32_out_t[30] = 1 ^ data_xor_f_reg[4] ^ data_xor_f_reg[7] ^ data_xor_f_reg[8] ^ data_xor_f_reg[10] ^ data_xor_f_reg[14] ^ data_xor_f_reg[22] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[26] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[32] ^ data_xor_f_reg[35] ^ data_xor_f_reg[42] ^ data_xor_f_reg[43] ^ data_xor_f_reg[45] ^ data_xor_f_reg[46] ^ data_xor_f_reg[48] ^ data_xor_f_reg[51] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[56] ^ data_xor_f_reg[58] ^ data_xor_f_reg[59] ^ data_xor_f_reg[61] ^ data_xor_f_reg[63];
        crc32_out_t[31] = 0 ^ data_xor_f_reg[5] ^ data_xor_f_reg[8] ^ data_xor_f_reg[9] ^ data_xor_f_reg[11] ^ data_xor_f_reg[15] ^ data_xor_f_reg[23] ^ data_xor_f_reg[24] ^ data_xor_f_reg[25] ^ data_xor_f_reg[27] ^ data_xor_f_reg[28] ^ data_xor_f_reg[29] ^ data_xor_f_reg[30] ^ data_xor_f_reg[31] ^ data_xor_f_reg[33] ^ data_xor_f_reg[36] ^ data_xor_f_reg[43] ^ data_xor_f_reg[44] ^ data_xor_f_reg[46] ^ data_xor_f_reg[47] ^ data_xor_f_reg[49] ^ data_xor_f_reg[52] ^ data_xor_f_reg[53] ^ data_xor_f_reg[54] ^ data_xor_f_reg[57] ^ data_xor_f_reg[59] ^ data_xor_f_reg[60] ^ data_xor_f_reg[62];
    end
   
endmodule