// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ps/1ps

module tb_top;
    import gdr_pkt_pkg::*;
   
      
    logic clk, clkf, rst, clk_avmm;
    
    always @(*) begin
	clkf <= #1000 !clkf;
    end
    
    always @(*) begin
	clk <= #2000 !clk;
    end
    
    always @(*) begin
	clk_avmm <= #4500 !clk_avmm;
    end
    
    
    localparam RX2TX_LB = 0;
    
    logic rx2tx_lb_en;
    
    
    
    //----------------------------------------------------------------------------------------
    // csr interface
    logic [9:0] address, address_0, address_1;
    logic [31:0] writedata, writedata_0, writedata_1;
    logic 	 read, read_0, read_1;
    logic 	 write, write_0, write_1;
    
    logic [31:0] readdata, readdata_0, readdata_1;
    logic 	 readdatavalid, readdatavalid_0, readdatavalid_1;
    logic 	 waitrequest, waitrequest_0, waitrequest_1;
    
    //----------------------------------------------------------------------------------------
    // tmii
    logic 	 tx_mii_ready;
    
    PCS_D_16_WRD_s tx_mii_d, tx_mii_d_0, tx_mii_d_1;
    PCS_C_16_WRD_s tx_mii_c, tx_mii_c_0, tx_mii_c_1;
    PCS_SYNC_16_WRD_s tx_mii_sync, tx_mii_sync_0, tx_mii_sync_1;
    logic 	 tx_mii_vld, tx_mii_vld_0, tx_mii_vld_1;
    logic 	 tx_mii_am, tx_mii_am_0, tx_mii_am_1;
    
    //----------------------------------------------------------------------------------------
    // rmii 
    PCS_D_16_WRD_s rx_mii_d, rx_mii_d_0, rx_mii_d_1;
    PCS_SYNC_16_WRD_s rx_mii_sync, rx_mii_sync_0, rx_mii_sync_1;
    PCS_C_16_WRD_s rx_mii_c, rx_mii_c_0, rx_mii_c_1;
    logic 	 rx_mii_vld, rx_mii_vld_0, rx_mii_vld_1;
    logic 	 rx_mii_am, rx_mii_am_0, rx_mii_am_1;
    
    CFG_MODE_0_s cfg_mode_reg_0, cfg_mode_1_reg_0;
    CFG_MODE_1_s cfg_mode_reg_1, cfg_mode_1_reg_1;
    CFG_MODE_2_s cfg_mode_reg_2, cfg_mode_1_reg_2;
    CFG_MODE_3_s cfg_mode_reg_3, cfg_mode_1_reg_3;
    CFG_START_PKT_GEN_s cfg_start_pkt_gen_reg, cfg_start_pkt_gen_1_reg;
    CFG_START_XFER_PKT_s cfg_xfer_pkt_reg, cfg_xfer_pkt_1_reg;
    CFG_CNT_STATS_s cnt_reg, cnt_1_reg;
    CFG_MEM_ACCESS_CTL_s mem_access_ctl_reg;
    CFG_MEM_MISC_s mem_misc_reg;
    
    logic 	 test_fail, test_fail_stats, test_fail_1, test_fail_stats_1;
    CFG_DREG_s readdata_reg, readdata_1_reg, cfg_data_reg;

    CFG_DREG_s [63:0] mem_data_array;
    CFG_DREG_s [7:0]  mem_ctl_array;
    
    //----------------------------------------------------------------------------------------
    //----------------------------------------------------------------------------------------
    // AM Insertion logic checker
    logic [3:0]  tb_am_ins_cyc, tb_am_ins_cyc_cnt, tb_am_ins_sample_cnt,
		 tb_am_ins_cnt, am_ins_chk_err_cnt;
    logic [16:0] tb_am_ins_period, tb_am_ins_period_cnt;
    logic 	 tb_am_ins_en, tb_tx_vld_end, tb_tx_vld_start,
		 tb_am_ins_cyc_en, 
                 tb_am_ins_cyc_end, tb_am_ins_cyc_start,
		 tb_tx_mii_am_pos, tb_tx_mii_am_neg, tb_start_am_ins_sample,
		 tx_mii_am_c1, am_ins_chk_err, am_ins_chk_err_stat;

    always_ff @(posedge clkf) begin
	if (tb_start_am_ins_sample & tx_mii_vld)
	    tb_am_ins_period_cnt <= tb_am_ins_period_cnt + 1'b1;

	if (tb_start_am_ins_sample & tx_mii_vld & 
	    (tb_am_ins_period_cnt ==  (tb_am_ins_period-1'b1) ) )
	     tb_am_ins_period_cnt <= '0;
	
	if (rst                    |
	    !tb_start_am_ins_sample |
            ( tb_am_ins_en & tx_mii_vld & 
	      (tb_am_ins_period_cnt > tb_am_ins_period))  )
	    tb_am_ins_period_cnt <= '0;	
    end

    always_comb begin
	tb_tx_vld_start =
	   (tb_am_ins_period_cnt == '0) & 
           tb_start_am_ins_sample & tx_mii_vld;
	
	tb_tx_vld_end = 
           (tb_am_ins_period_cnt ==  (tb_am_ins_period-1'b1)) & 
           tb_start_am_ins_sample & tx_mii_vld;	
    end

    always_ff @(posedge clkf) begin
	tx_mii_am_c1 <= tx_mii_am;
    end

    always_comb begin
	tb_tx_mii_am_pos = !tx_mii_am_c1 & tx_mii_am;
	tb_tx_mii_am_neg = tx_mii_am_c1 & !tx_mii_am;
    end

    always_ff @(posedge clkf) begin
	if (tb_tx_mii_am_pos & (tb_am_ins_sample_cnt != 4'd3))
	    tb_am_ins_sample_cnt <= tb_am_ins_sample_cnt + 1'b1;

	if ( (tb_am_ins_sample_cnt == 4'd3) & tb_tx_mii_am_pos 
	     & !tb_start_am_ins_sample)
	    tb_start_am_ins_sample <= '1;
	
	if (rst           | 
	    !tb_am_ins_en   ) begin
	    tb_am_ins_sample_cnt          <= '0;
	    tb_start_am_ins_sample <= '0;
	end
    end // always_ff @ (posedge clkf)


    always_ff @(posedge clkf) begin
	if (tb_start_am_ins_sample & tb_tx_vld_end)
	     tb_am_ins_cyc_en <= '1;

	if (tb_start_am_ins_sample & tb_am_ins_cyc_end)
	    tb_am_ins_cyc_en <= '0;

	if (rst                     |
	    !tb_start_am_ins_sample  )
	    tb_am_ins_cyc_en <= '0;
	
    end

    always_ff @(posedge clkf) begin
	if (tb_am_ins_cyc_en)
	    tb_am_ins_cyc_cnt <= tb_am_ins_cyc_cnt + 1'b1;

	if (tb_am_ins_cyc_end)
	    tb_am_ins_cyc_cnt <= '0;
	
	if (rst)
	    tb_am_ins_cyc_cnt <= '0;	
    end // always_ff @ (posedge clkf)

    always_comb begin
	tb_am_ins_cyc_end = 
          (tb_am_ins_cyc_cnt  == (tb_am_ins_cyc - 1'b1) ) & 
	  tb_start_am_ins_sample;	
    end // always_comb

    always_ff @(posedge clkf) begin
	am_ins_chk_err <= (tb_am_ins_cyc_en != tx_mii_am) &
			  tb_start_am_ins_sample ;

	if (am_ins_chk_err & (am_ins_chk_err_cnt > 4'd1) )
	    am_ins_chk_err_stat <= '1;

	if (am_ins_chk_err & (am_ins_chk_err_cnt != '1))
	    am_ins_chk_err_cnt <= am_ins_chk_err_cnt + 1'b1;
	
	if (rst |
	    !tb_start_am_ins_sample) begin
	    am_ins_chk_err <= '0;
	    am_ins_chk_err_stat <= '0;
	    am_ins_chk_err_cnt <= '0;
	    
	end
    end
    
    always_ff @(posedge clkf) begin
	if (am_ins_chk_err & (am_ins_chk_err_cnt > 4'd1) ) begin
	    $display("**** AM Insertion Error **** ");	    
	end
    end
    //----------------------------------------------------------------------------------------
    //----------------------------------------------------------------------------------------
    
    always_ff @(posedge clk_avmm) begin
	if (test_fail)
	    test_fail_stats <= '1;
	
	if (rst)
	    test_fail_stats <= '0;
    end
    
    always_ff @(posedge clk_avmm) begin
	if (test_fail_1)
	    test_fail_stats_1 <= '1;
	
	if (rst)
	    test_fail_stats_1 <= '0;
    end
    
    initial begin
	$vcdpluson;	 
	Init;

	test_csr_gen_pkt ;
	
       		
	repeat (10) @(posedge clk);
        
	$finish;
    end // initial begin

    task test_csr_gen_pkt;

	begin
	    repeat (200) @(posedge clk_avmm);	 
	    $display("***********************************************************************");
	    $display("**** Test: CSR GEN_PKT MODE_PCS; MODE_400G;                        ****");
	    $display("***********************************************************************");
	    test_csr_gen_pkt_lb (MODE_OP_e'(MODE_PCS), 
		       MODE_e'(MODE_400G), 
		       PKT_LEN_MODE_e'(INC_PKT_LEN),
		       8'd0,         // no_of_inc_bytes  
		       11'd0,        // fix_pkt_len
		       13'd0,        // no_of_pkt_gen	     
		       16'd500       // no_of_xfer_pkt
		       
		   );
	end
    endtask // test_csr_gen_pkt

    task test_csr_gen_pkt_lb;
	input MODE_OP_e      mode_op;
	input MODE_e         mode;
	input PKT_LEN_MODE_e pkt_len_mode;
	input [7:0] no_of_inc_bytes;
	input [10:0] fix_pkt_len;
	input [12:0] no_of_pkt_gen;      
	input [15:0] no_of_xfer_pkt;
	
	logic disable_am_ins;
	logic [3:0]  am_ins_cyc;
	logic [16:0] am_ins_period;
	begin
	    
	    disable_am_ins = '1;
	    am_ins_cyc = '0;
	    am_ins_period = '0;
	    
	    rx2tx_lb_en = '1;
	    
	    sys_reset;
	    
	    cfg_dev_0 ( mode_op,
			mode,
			pkt_len_mode,
       			no_of_inc_bytes,
			fix_pkt_len,
       			no_of_pkt_gen,      
			no_of_xfer_pkt,
			!rx2tx_lb_en,         
			disable_am_ins,
			am_ins_cyc,
			am_ins_period
			);
	    
	    cfg_dev_1 ( mode_op,
			mode,
			pkt_len_mode,
       			no_of_inc_bytes,
			fix_pkt_len,
       			no_of_pkt_gen,      
			no_of_xfer_pkt,
			rx2tx_lb_en,         
			disable_am_ins,
			am_ins_cyc,
			am_ins_period
			);
	    
	    
	    repeat (10) @(posedge clk_avmm);
	    
	    // generate pkts "write pkt data to buffer memory"
	    csr_gen_pkt ;
	    
	    $display("*********************************************************************** ");
	    $display("****        Read Counters Registers Before Pkts Transmission       **** ");
	    // read counters before xfer pkts
	    csr_rd_cnt;
	    csr_rd_cnt_1;
	    
	    repeat (20) @(posedge clk_avmm);
	    // read cfg registers before xfer pkts
	    $display("*********************************************************************** ");
	    $display("****       Read Config Registers Before Pkts Transmission          **** ");
	    csr_rd_cfg_reg;
	    csr_rd_cfg_reg_1;
	    repeat (20) @(posedge clk_avmm);
	    
	    // transfer pkt
	    xfer_pkt (no_of_xfer_pkt, 32'd0);
	end
    endtask

    
    task csr_gen_pkt;
	begin
	    CFG_DREG_s [63:0]  mem_data_wrd;
	    CFG_DREG_s [7:0]   mem_ctl_wrd;
	    CFG_START_PKT_GEN_s csr_gen_pkt_reg;
	    
	    logic [63:0] [3:0]  ctl_wrd;
	    
	    logic mem_sop, mem_terminate;
	    
	    logic [1:0] mem_misc;
	    
	    repeat (200) @(posedge clk_avmm);	 
	    $display("***********************************************************************");
	    $display("**** Start CSR Generated Packet                                    ****");
	    
	    //-------------------------------------------------------------------------------
	    // generate 64 bytes
	    for (int b = 0; b < 64 ; b ++) begin
		// sop
		if (b == 0) begin
		    mem_data_wrd[b] = PCS_SOP_DATA;
		    ctl_wrd[b] = 4'b1000;		    
		end
		// sfd
		else if ( b == 1) begin
		    mem_data_wrd[b] = PCS_PRE_DATA;
		    ctl_wrd[b] = 4'b0000;
		end
		// data
		else if ( b  < 17)  begin
		    mem_data_wrd[b][31:16] = ((b-2)*2);
		    mem_data_wrd[b][15:0]  = ((b-2)*2) + 1;
		    ctl_wrd[b] = 4'b0000;
		end
		// crc
		else if ( b == 17) begin
		    mem_data_wrd[b] = 32'hc984_9626;
		    ctl_wrd[b] = 4'b0000;
		end
		// terminate
		else if (b == 18) begin
		    mem_data_wrd[b] = PCS_TERMINATE_DATA;
		    ctl_wrd[b] = 4'b1111;
		end
		// idle
		else begin
		    mem_data_wrd[b] = PCS_IDLE_DATA;
		    ctl_wrd[b] = 4'b1111;
		end				      
	    end

	    for (int a = 0; a < 8; a ++) begin
		mem_ctl_wrd[a] = {ctl_wrd[(a*8)+7],
				  ctl_wrd[(a*8)+6],
				  ctl_wrd[(a*8)+5],
				  ctl_wrd[(a*8)+4],
				  ctl_wrd[(a*8)+3],
				  ctl_wrd[(a*8)+2],
				  ctl_wrd[(a*8)+1],
				  ctl_wrd[(a*8)] };		                 
	    end // for (int a = 0; a < 8; a ++)

	    mem_sop = '1;
	    mem_terminate = '1;

	    mem_acc_wr('d0, mem_data_wrd, mem_ctl_wrd, mem_sop, mem_terminate);
	    //--------------------------------------------------------------------------------
	   
	    //-------------------------------------------------------------------------------
	    // generate 65 bytes
	    for (int b = 0; b < 64 ; b ++) begin
		// sop
		if (b == 0) begin
		    mem_data_wrd[b] = PCS_SOP_DATA;
		    ctl_wrd[b] = 4'b1000;		    
		end
		// sfd
		else if ( b == 1) begin
		    mem_data_wrd[b] = PCS_PRE_DATA;
		    ctl_wrd[b] = 4'b0000;
		end
		// data
		else if ( b  < 17)  begin
		    mem_data_wrd[b][31:16] = ((b-2)*2);
		    mem_data_wrd[b][15:0]  = ((b-2)*2) + 1;
		    ctl_wrd[b] = 4'b0000;
		end
		// {data, crc[3:1]
		else if ( b == 17) begin
		    mem_data_wrd[b] = 32'h00_c6ee;
		    ctl_wrd[b] = 4'b0000;
		end
		// crc[0], terminate
		else if (b == 18) begin
		    mem_data_wrd[b] = {8'he6, PCS_C_END, PCS_C_IDLE, PCS_C_IDLE};
		    ctl_wrd[b] = 4'b0111;
		end
		// idle
		else begin
		    mem_data_wrd[b] = PCS_IDLE_DATA;
		    ctl_wrd[b] = 4'b1111;
		end				      
	    end

	    for (int a = 0; a < 8; a ++) begin
		mem_ctl_wrd[a] = {ctl_wrd[(a*8)+7],
				  ctl_wrd[(a*8)+6],
				  ctl_wrd[(a*8)+5],
				  ctl_wrd[(a*8)+4],
				  ctl_wrd[(a*8)+3],
				  ctl_wrd[(a*8)+2],
				  ctl_wrd[(a*8)+1],
				  ctl_wrd[(a*8)] };		                 
	    end // for (int a = 0; a < 8; a ++)

	    mem_sop = '1;
	    mem_terminate = '1;

	    mem_acc_wr('d1, mem_data_wrd, mem_ctl_wrd, mem_sop, mem_terminate);
	    //--------------------------------------------------------------------------------

	    
	    //-------------------------------------------------------------------------------
	    // generate 256 bytes
	    for (int b = 0; b < 64 ; b ++) begin
		// sop
		if (b == 0) begin
		    mem_data_wrd[b] = PCS_SOP_DATA;
		    ctl_wrd[b] = 4'b1000;		    
		end
		// sfd
		else if ( b == 1) begin
		    mem_data_wrd[b] = PCS_PRE_DATA;
		    ctl_wrd[b] = 4'b0000;
		end
		// data
		else  begin
		    mem_data_wrd[b][31:16] = ((b-2)*2);
		    mem_data_wrd[b][15:0]  = ((b-2)*2) + 1;
		    ctl_wrd[b] = 4'b0000;
		end					      
	    end

	    for (int a = 0; a < 8; a ++) begin
		mem_ctl_wrd[a] = {ctl_wrd[(a*8)+7],
				  ctl_wrd[(a*8)+6],
				  ctl_wrd[(a*8)+5],
				  ctl_wrd[(a*8)+4],
				  ctl_wrd[(a*8)+3],
				  ctl_wrd[(a*8)+2],
				  ctl_wrd[(a*8)+1],
				  ctl_wrd[(a*8)] };		                 
	    end // for (int a = 0; a < 8; a ++)

	    mem_sop = '1;
	    mem_terminate = '0;

	    mem_acc_wr('d2, mem_data_wrd, mem_ctl_wrd, mem_sop, mem_terminate);

	    for (int b = 0; b < 64 ; b ++) begin
		// data
		if (b == 0) begin
		    mem_data_wrd[b] = 32'h007c_007d;
		    ctl_wrd[b]      = 4'b0000;
		end
		// crc
		else if (b == 1) begin
		    mem_data_wrd[b] = 32'h54d3_185e;
		    ctl_wrd[b]      = 4'b0000;
		end
		// terminate
		else if (b == 2) begin
		    mem_data_wrd[b] = PCS_TERMINATE_DATA;
		    ctl_wrd[b] = 4'b1111;
		end
		// idle
		else begin
		    mem_data_wrd[b] = PCS_IDLE_DATA;
		    ctl_wrd[b] = 4'b1111;
		end // else: !if(b == 2)
	    end // for (int b = 0; b < 64 ; b ++)

	    for (int a = 0; a < 8; a ++) begin
		mem_ctl_wrd[a] = {ctl_wrd[(a*8)+7],
				  ctl_wrd[(a*8)+6],
				  ctl_wrd[(a*8)+5],
				  ctl_wrd[(a*8)+4],
				  ctl_wrd[(a*8)+3],
				  ctl_wrd[(a*8)+2],
				  ctl_wrd[(a*8)+1],
				  ctl_wrd[(a*8)] };		                 
	     end // for (int a = 0; a < 8; a ++)
	    
	    mem_sop = '0;
	    mem_terminate = '1;
	    mem_acc_wr('d3, mem_data_wrd, mem_ctl_wrd, mem_sop, mem_terminate);
	    //--------------------------------------------------------------------------------

	    // write to CFG_START_PKT_GEN register to indicates sw_gen_pkt and cfg_last_mem_addr
	    csr_gen_pkt_reg = '0;
	    csr_gen_pkt_reg.sw_gen_pkt = '1;
	    csr_gen_pkt_reg.last_mem_addr = 13'd3;
	    
	    csr_wr_access( {ADDR_CFG_START_PKT_GEN, 2'd0}, csr_gen_pkt_reg);
	    
	    repeat (100) @(posedge clk_avmm);
	    $display("**** CSR Generated Packet Done                                     ****");
	    $display("***********************************************************************");
	   
	    repeat (10) @(posedge clk_avmm);
	    
       end
    endtask // test_am_PCS_400G_fix_pkt_len

    task mem_acc_wr;
	input [15:0] wr_addr;
	input 	     CFG_DREG_s [63:0]  mem_data_wrd;
	input 	     CFG_DREG_s [7:0]   mem_ctl_wrd;
	input 	     mem_sop;
	input 	     mem_terminate;

	begin
	    CFG_MEM_ACCESS_CTL_s mem_acc_ctl_rd_reg, mem_acc_ctl_wr_reg;
	    CFG_MEM_MISC_s mem_misc_data_reg;
	    
	    logic [7:0] mem_addr;
	    
	    mem_acc_ctl_wr_reg = '0;
	    mem_acc_ctl_wr_reg.addr_req = wr_addr;
	    mem_acc_ctl_wr_reg.wr_req =  '1;	 

	    mem_misc_data_reg = '0;
	    mem_misc_data_reg.sop = mem_sop;
	    mem_misc_data_reg.terminate = mem_terminate;
	    

	    $display("**** Write Pkt Memory Addr=%0d ****", wr_addr);
	    
	    // write to mem_data_0 to mem_data_63 registers
	    for (int i = 0; i < 64; i++) begin
		mem_addr = ADDR_MEM_DATA_0 + i;
		csr_wr_access( {mem_addr, 2'd0}, mem_data_wrd[i]);
	    end

	    // write to mem_ctl_0 to mem_ctl_7 registers
	    for (int i = 0; i < 8; i++) begin
		mem_addr =ADDR_MEM_CTL_0  + i;
		csr_wr_access( {mem_addr, 2'd0}, mem_ctl_wrd[i]);
	    end

	    // write to mem_misc register
	    csr_wr_access( {ADDR_MEM_MISC, 2'd0}, mem_misc_data_reg );
	    
	    // write request to mem
	    csr_wr_access( {ADDR_MEM_ACCESS, 2'd0}, mem_acc_ctl_wr_reg);
	    repeat (4) @(posedge clk_avmm);
	    // polling for read request done
	    csr_rd_access ({ADDR_MEM_ACCESS, 2'd0});

	    while (mem_access_ctl_reg.wr_req) begin
		csr_rd_access ({ADDR_MEM_ACCESS, 2'd0});
		repeat (50) @(posedge clk_avmm);
	    end
	end
    endtask // mem_acc_wr


    task mem_acc_rd_exp;
	input [15:0] rd_addr;
	input 	     CFG_DREG_s [63:0]  mem_data_wrd_exp;
	input 	     CFG_DREG_s [7:0]   mem_ctl_wrd_exp;
	input 	     mem_sop_exp;
	input 	     mem_terminate_exp;
	
	
	
	begin
	    CFG_MEM_ACCESS_CTL_s mem_acc_ctl_rd_reg, mem_acc_ctl_wr_reg;
	    CFG_MEM_MISC_s mem_misc_reg_exp;
	    
	    mem_acc_ctl_wr_reg = '0;
	    mem_acc_ctl_wr_reg.addr_req = rd_addr;
	    mem_acc_ctl_wr_reg.rd_req =  '1;

	    mem_misc_reg_exp = '0;
	    mem_misc_reg_exp.sop = mem_sop_exp;
	    mem_misc_reg_exp.terminate = mem_terminate_exp;
	    
	    $display("**** Read Pkt Memory Addr=%0d ****", rd_addr);
	    
	    // read request to mem
	    csr_wr_access( {ADDR_MEM_ACCESS, 2'd0}, mem_acc_ctl_wr_reg);
	    repeat (4) @(posedge clk_avmm);

	    // polling for read request done
	    csr_rd_access ({ADDR_MEM_ACCESS, 2'd0});

	    while (mem_access_ctl_reg.rd_req) begin
		csr_rd_access ({ADDR_MEM_ACCESS, 2'd0});
		repeat (50) @(posedge clk_avmm);
	    end
	    for (int i = 0 ; i < 64; i++) begin
		csr_rd_access_exp({{ADDR_MEM_DATA_0+i}, 2'd0}, mem_data_wrd_exp[i]);
		repeat (2) @(posedge clk_avmm);
	    end
	    for (int i = 0 ; i < 8; i++) begin
		csr_rd_access_exp({{ADDR_MEM_CTL_0+i}, 2'd0}, mem_ctl_wrd_exp[i]);
		repeat (2) @(posedge clk_avmm);
	    end
	    csr_rd_access_exp({ADDR_MEM_MISC, 2'd0}, mem_misc_reg_exp);
	    
	    $display("**** Read Pkt Memory Done ****");
	end
    endtask
    
    task mem_acc_rd;
	input [15:0] rd_addr;

	CFG_MEM_ACCESS_CTL_s mem_acc_ctl_rd_reg, mem_acc_ctl_wr_reg;
	
	begin
	    mem_acc_ctl_wr_reg = '0;
	    mem_acc_ctl_wr_reg.addr_req = rd_addr;
	    mem_acc_ctl_wr_reg.rd_req =  '1;

	    $display("**** Read Pkt Memory Addr=%0d ****", rd_addr);
	    
	    // read request to mem
	    csr_wr_access( {ADDR_MEM_ACCESS, 2'd0}, mem_acc_ctl_wr_reg);
	    repeat (4) @(posedge clk_avmm);

	    // polling for read request done
	    csr_rd_access ({ADDR_MEM_ACCESS, 2'd0});

	    while (mem_access_ctl_reg.rd_req) begin
		csr_rd_access ({ADDR_MEM_ACCESS, 2'd0});
		repeat (50) @(posedge clk_avmm);
	    end
	    for (int i = 0 ; i < 64; i++) begin
		csr_rd_access({{ADDR_MEM_DATA_0+i}, 2'd0});
		repeat (2) @(posedge clk_avmm);
	    end
	    for (int i = 0 ; i < 8; i++) begin
		csr_rd_access({{ADDR_MEM_CTL_0+i}, 2'd0});
		repeat (2) @(posedge clk_avmm);
	    end
	    csr_rd_access({ADDR_MEM_MISC, 2'd0});
	    
	    $display("**** Read Pkt Memory Done ****");
	end
    endtask
    

    //----------------------------------------------------------------------------------------
    // AM insertion
    task test_am_lb;
	input MODE_OP_e      mode_op;
	input MODE_e         mode;
	input PKT_LEN_MODE_e pkt_len_mode;
	input [7:0] no_of_inc_bytes;
	input [10:0] fix_pkt_len;
	input [12:0] no_of_pkt_gen;      
	input [15:0] no_of_xfer_pkt;	
	input disable_am_ins;
	input [3:0]  am_ins_cyc;
	input [16:0] am_ins_period;
	
	begin	    
	    rx2tx_lb_en = '1;
	    tb_am_ins_en     = '1;
	    tb_am_ins_cyc    = am_ins_cyc;
	    tb_am_ins_period = am_ins_period ;
	    
	    sys_reset;
	    
	    cfg_dev_0 ( mode_op,
			mode,
			pkt_len_mode,
       			no_of_inc_bytes,
			fix_pkt_len,
       			no_of_pkt_gen,      
			no_of_xfer_pkt,
			!rx2tx_lb_en,         
			disable_am_ins,
			am_ins_cyc,
			am_ins_period
			);
	    
	    cfg_dev_1 ( mode_op,
			mode,
			pkt_len_mode,
       			no_of_inc_bytes,
			fix_pkt_len,
       			no_of_pkt_gen,      
			no_of_xfer_pkt,
			rx2tx_lb_en,         
			'1,
			am_ins_cyc,
			am_ins_period
			);
	    
	    
	    repeat (10) @(posedge clk_avmm);
	    
	    // generate pkts "write pkt data to buffer memory"
	    gen_pkt;
	    
	    $display("*********************************************************************** ");
	    $display("****        Read Counters Registers Before Pkts Transmission       **** ");
	    // read counters before xfer pkts
	    csr_rd_cnt;
	    csr_rd_cnt_1;
	    
	    repeat (20) @(posedge clk_avmm);
	    // read cfg registers before xfer pkts
	    $display("*********************************************************************** ");
	    $display("****       Read Config Registers Before Pkts Transmission          **** ");
	    csr_rd_cfg_reg;
	    csr_rd_cfg_reg_1;
	    repeat (20) @(posedge clk_avmm);
	    
	    // transfer pkt
	    xfer_pkt (no_of_xfer_pkt, 32'd0);
	end
    endtask // test_lb

    
    task test_lb;
	input MODE_OP_e      mode_op;
	input MODE_e         mode;
	input PKT_LEN_MODE_e pkt_len_mode;
	input [7:0] no_of_inc_bytes;
	input [10:0] fix_pkt_len;
	input [12:0] no_of_pkt_gen;      
	input [15:0] no_of_xfer_pkt;
	
	logic disable_am_ins;
	logic [3:0]  am_ins_cyc;
	logic [16:0] am_ins_period;
	begin
	    
	    disable_am_ins = '1;
	    am_ins_cyc = '0;
	    am_ins_period = '0;
	    
	    rx2tx_lb_en = '1;
	    
	    sys_reset;
	    
	    cfg_dev_0 ( mode_op,
			mode,
			pkt_len_mode,
       			no_of_inc_bytes,
			fix_pkt_len,
       			no_of_pkt_gen,      
			no_of_xfer_pkt,
			!rx2tx_lb_en,         
			disable_am_ins,
			am_ins_cyc,
			am_ins_period
			);
	    
	    cfg_dev_1 ( mode_op,
			mode,
			pkt_len_mode,
       			no_of_inc_bytes,
			fix_pkt_len,
       			no_of_pkt_gen,      
			no_of_xfer_pkt,
			rx2tx_lb_en,         
			disable_am_ins,
			am_ins_cyc,
			am_ins_period
			);
	    
	    
	    repeat (10) @(posedge clk_avmm);
	    
	    // generate pkts "write pkt data to buffer memory"
	    gen_pkt;
	    
	    $display("*********************************************************************** ");
	    $display("****        Read Counters Registers Before Pkts Transmission       **** ");
	    // read counters before xfer pkts
	    csr_rd_cnt;
	    csr_rd_cnt_1;
	    
	    repeat (20) @(posedge clk_avmm);
	    // read cfg registers before xfer pkts
	    $display("*********************************************************************** ");
	    $display("****       Read Config Registers Before Pkts Transmission          **** ");
	    csr_rd_cfg_reg;
	    csr_rd_cfg_reg_1;
	    repeat (20) @(posedge clk_avmm);
	    
	    // transfer pkt
	    xfer_pkt (no_of_xfer_pkt, 32'd0);
	end
    endtask // test_lb
   
    // Config Device 0
    task cfg_dev_0;
	input MODE_OP_e      mode_op;
	input MODE_e         mode;
	input PKT_LEN_MODE_e pkt_len_mode;
	input [7:0] no_of_inc_bytes;
	input [10:0] fix_pkt_len;
	input [12:0] no_of_pkt_gen;      
	input [15:0] no_of_xfer_pkt;
	input 	     rx2tx_lb;
	input 	     disable_am_ins;
	input [3:0]  am_ins_cyc;
	input [16:0] am_ins_period;
	
	CFG_MODE_0_s cfg_mode_reg_0;
	CFG_MODE_1_s cfg_mode_reg_1;
	CFG_MODE_2_s cfg_mode_reg_2;
	CFG_MODE_3_s cfg_mode_reg_3;
	
	begin
	    $display("***********************************************************************");
	    $display("****            Read Config Registers Default Settings             ****\n");
	    csr_rd_cfg_reg;
	    
	    repeat (20) @(posedge clk_avmm);
	    
	    $display("***********************************************************************");
	    $display("**** Start Config Registers: Rd Modify Wr to Cfg Register          ****");	    
	    csr_rd_access({ADDR_CFG_MODE_3, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    cfg_mode_reg_3 = readdata_reg;
	    cfg_mode_reg_3.am_ins_cyc = am_ins_cyc;
	    cfg_mode_reg_3.am_ins_period = am_ins_period;
	    // set am info
	    csr_wr_access ({ADDR_CFG_MODE_3, 2'd0}, cfg_mode_reg_3);
	    repeat (2) @(posedge clk_avmm);

	    
	    csr_rd_access({ADDR_CFG_MODE_2, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    cfg_mode_reg_2 = readdata_reg;
	    cfg_mode_reg_2.no_of_pkt_gen  = no_of_pkt_gen;
	    cfg_mode_reg_2.no_of_xfer_pkt = no_of_xfer_pkt;
	    // set number of pkt to transmit
	    csr_wr_access ({ADDR_CFG_MODE_2, 2'd0}, cfg_mode_reg_2);
	    repeat (2) @(posedge clk_avmm);
	    
	    csr_rd_access({ADDR_CFG_MODE_1, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    cfg_mode_reg_1 = readdata_reg;
	    cfg_mode_reg_1.fix_pkt_len = fix_pkt_len;
	    cfg_mode_reg_1.no_of_inc_bytes = no_of_inc_bytes;	 
	    // set fix_pkt_len and no_of_inc_bytes
	    csr_wr_access ({ADDR_CFG_MODE_1, 2'd0}, cfg_mode_reg_1);
	    repeat (2) @(posedge clk_avmm);
	    
	    csr_rd_access({ADDR_CFG_MODE_0, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    cfg_mode_reg_0 = readdata_reg;
	    cfg_mode_reg_0.mode           = mode;
	    cfg_mode_reg_0.mode_op        = mode_op;
	    cfg_mode_reg_0.pkt_len_mode   = pkt_len_mode;
	    cfg_mode_reg_0.rx2tx_lb       = rx2tx_lb;
	    cfg_mode_reg_0.disable_am_ins = disable_am_ins;
	    // set mode of operation
	    csr_wr_access ({ADDR_CFG_MODE_0, 2'd0}, cfg_mode_reg_0);
	    repeat (10) @(posedge clk_avmm);
	    $display("****                     Config Registers Done                     ****");
	    $display("***********************************************************************\n");
	end
    endtask
	
    // Config Device 1
    task cfg_dev_1;
	input MODE_OP_e      mode_op;
	input MODE_e         mode;
	input PKT_LEN_MODE_e pkt_len_mode;
	input [7:0] no_of_inc_bytes;
	input [10:0] fix_pkt_len;
	input [12:0] no_of_pkt_gen;      
	input [15:0] no_of_xfer_pkt;
	input 	     rx2tx_lb;
	input 	     disable_am_ins;
	input [3:0]  am_ins_cyc;
	input [16:0] am_ins_period;
       	
	CFG_MODE_0_s t1_cfg_mode_0;
	CFG_MODE_1_s t1_cfg_mode_1;
	CFG_MODE_2_s t1_cfg_mode_2;
	CFG_MODE_3_s t1_cfg_mode_3;
	
	begin
	    $display("***********************************************************************");
	    $display("****        Dev_1 Read Config Registers Default Settings           ****\n");
	    csr_rd_cfg_reg_1;
	    
	    repeat (20) @(posedge clk_avmm);
	    
	    $display("***********************************************************************");
	    $display("**** Dev_1 Start Config Registers: Rd Modify Wr to Cfg Register    ****");
	    csr_rd_access_1({ADDR_CFG_MODE_3, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    t1_cfg_mode_3 = readdata_1_reg;
	    t1_cfg_mode_3.am_ins_cyc = am_ins_cyc;
	    t1_cfg_mode_3.am_ins_period = am_ins_period;
	    // set am info
	    csr_wr_access_1 ({ADDR_CFG_MODE_3, 2'd0}, t1_cfg_mode_3);
	    repeat (2) @(posedge clk_avmm);
	    
	    csr_rd_access_1({ADDR_CFG_MODE_2, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    t1_cfg_mode_2 = readdata_1_reg;
	    t1_cfg_mode_2.no_of_pkt_gen  = no_of_pkt_gen;
	    t1_cfg_mode_2.no_of_xfer_pkt = no_of_xfer_pkt;
	    // set number of pkt to transmit
	    csr_wr_access_1 ({ADDR_CFG_MODE_2, 2'd0}, t1_cfg_mode_2);
	    repeat (2) @(posedge clk_avmm);
	    
	    csr_rd_access_1({ADDR_CFG_MODE_1, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    t1_cfg_mode_1 = readdata_1_reg;
	    t1_cfg_mode_1.fix_pkt_len = fix_pkt_len;
	    t1_cfg_mode_1.no_of_inc_bytes = no_of_inc_bytes;	 
	    // set fix_pkt_len and no_of_inc_bytes
	    csr_wr_access_1 ({ADDR_CFG_MODE_1, 2'd0}, t1_cfg_mode_1);
	    repeat (2) @(posedge clk_avmm);
	    
	    csr_rd_access_1({ADDR_CFG_MODE_0, 2'd0});
	    repeat (2) @(posedge clk_avmm);
	    t1_cfg_mode_0 = readdata_1_reg;
	    t1_cfg_mode_0.mode           = mode;
	    t1_cfg_mode_0.mode_op        = mode_op;
	    t1_cfg_mode_0.pkt_len_mode   = pkt_len_mode;
	    t1_cfg_mode_0.rx2tx_lb       = rx2tx_lb;
	    t1_cfg_mode_0.disable_am_ins = disable_am_ins;
	    
	    // set mode of operation
	    csr_wr_access_1 ({ADDR_CFG_MODE_0, 2'd0}, t1_cfg_mode_0);
	    repeat (10) @(posedge clk_avmm);
	    $display("****                Dev_1 Config Registers Done                    ****");
	    $display("***********************************************************************\n");
	end
    endtask

	
    task sys_reset;
	begin	    
	    rst = '1;	 
	    
	    repeat (100) @(posedge clk_avmm);
	    rst = '0;
	    repeat (1000) @(posedge clk_avmm);
	    
	    $display("***********************************************************************");
	    $display("****                   System Reset                                ****\n");
	    
	    $display("***********************************************************************");
	    $display("****                   Wait for Initialization Complete            ****");

	    repeat (7000) @(posedge clk_avmm);
	    
	    csr_rd_access( {ADDR_CFG_MODE_0, 2'd0} );
	    
	    while (!cfg_mode_reg_0.init_done) begin
		csr_rd_access( {ADDR_CFG_MODE_0, 2'd0} );
		repeat (2000) @(posedge clk_avmm);
		
	    end
	    
	    $display("****                   Initialization Complete                     ****");
	    $display("***********************************************************************\n");
	end
    endtask
   
   task Init;
       begin
	   clk  = '0;
	   clkf = '1;
	   clk_avmm = '0;
	   test_fail = '0;
	   rx2tx_lb_en = '0;
	   
	   address = '0;
	   writedata = '0;
	   read = '0;
	   write = '0;
	   
	   address_1 = '0;
	   writedata_1 = '0;
	   read_1 = '0;
	   write_1 = '0;

	   tb_am_ins_cyc = '0;
	   tb_am_ins_period = '0;
	   tb_am_ins_en = '0;
	   
	   //sys_reset;
	   
	 

	   repeat (1000) @(posedge clk);
       end
   endtask // Init

   logic [15:0] tx_mii_rdy_cnt;
   
   always_ff @(posedge clkf) begin
      tx_mii_rdy_cnt <= tx_mii_rdy_cnt + 1'b1;
      
      if (rst)
	tx_mii_rdy_cnt <= '0;
   end

   always_ff @(posedge clkf) begin
      if (tx_mii_rdy_cnt[7:2] == 6'h20)
	tx_mii_ready <= '0;
      else
	tx_mii_ready <= '1;
   end

    //-----------------------------------------------------------------------------------------
    // Packets Transmission
    task xfer_pkt;
	input [31:0] cnt_exp;
	input [31:0] err_cnt_exp;
	
	begin
	    CFG_START_XFER_PKT_s xfer_pkt_gen_reg;
	    CFG_MODE_2_s mode_2_reg;
	    
	    //csr_rd_cfg_reg;
	    repeat (100) @(posedge clk_avmm);
	    
	    xfer_pkt_gen_reg = '0;
	    xfer_pkt_gen_reg.start_xfer_pkt = '1;
	    $display("***********************************************************************");
	    $display("****             Start Packet Transmission                         ****");
	    
	    // initiate pkt transmission
	    csr_wr_access ({ADDR_CFG_START_XFER_PKT, 2'd0}, xfer_pkt_gen_reg);
	    
	    repeat (100) @(posedge clk_avmm);
	    csr_rd_access( {ADDR_CFG_START_XFER_PKT, 2'd0});
	    
	    while (!cfg_xfer_pkt_reg.pkt_xfer_done_stat) begin
		csr_rd_access( {ADDR_CFG_START_XFER_PKT, 2'd0} );
		repeat (5000) @(posedge clk_avmm);
	    end
	    
	    repeat (500) @(posedge clk_avmm);
	    $display("****               Packet Transmission Finish                      ****");
	    $display("***********************************************************************\n");
	    
	    repeat (1000) @(posedge clk_avmm);
	    // read dev_0 expected counters stats
	    csr_rd_cnt_exp(cnt_exp, err_cnt_exp);
	    
	    // read dev_1 expected counters stats
	    csr_rd_cnt_exp_1(cnt_exp, err_cnt_exp);	 
            
	    //csr_rd_cnt;
	    repeat (200) @(posedge clk_avmm);
	    
	    // check fail/pass stat
	    if (test_fail_stats | am_ins_chk_err_stat) begin
		if (test_fail_stats)
		    $display("**** Stats Check Error  ****");
		else if (am_ins_chk_err_stat)
		    $display("**** AM Insertion Check Error  ****");
		
		$display("**** Test FAILED **** \n");
		repeat (200) @(posedge clk_avmm);
		$finish;
	    end
	    else
		$display("**** Test PASSED **** \n");
	end
    endtask


   
   // packet generation
   task gen_pkt;

      begin
	 CFG_START_PKT_GEN_s start_pkt_gen_reg;
	 start_pkt_gen_reg = '0;
	 start_pkt_gen_reg.start_pkt_gen = '1;

	 $display("***********************************************************************");
	 $display("****                   Start Packet Generation                     ****");
	 csr_wr_access ({ADDR_CFG_START_PKT_GEN, 2'd0}, start_pkt_gen_reg);
	 repeat (100) @(posedge clk_avmm);
	 csr_rd_access( {ADDR_CFG_START_PKT_GEN, 2'd0} );
	 
	 while (!cfg_start_pkt_gen_reg.pkt_gen_done_stat) begin
	    csr_rd_access( {ADDR_CFG_START_PKT_GEN, 2'd0} );
	    repeat (10000) @(posedge clk_avmm);
	 end

	 $display("****              Packet Generation Finish                         ****");
	 $display("***********************************************************************\n");
      end
   endtask

   //-----------------------------------------------------------------------------------------
   // csr read config registers from exd_0
   task csr_rd_cfg_reg;
      begin
	 $display("**** Read Config Registers **** ");
	 csr_rd_access( {ADDR_CFG_MODE_0, 2'd0});
	 csr_rd_access( {ADDR_CFG_MODE_1, 2'd0});
	 csr_rd_access( {ADDR_CFG_MODE_2, 2'd0});
	 csr_rd_access( {ADDR_CFG_MODE_3, 2'd0});
	 $display(" \n ");
      end
   endtask // csr_rd_cfg_reg

   //-----------------------------------------------------------------------------------------
   // csr read config registers from exd_1
   task csr_rd_cfg_reg_1;
      begin
	 $display("**** Dev_1 Read Config Registers **** ");
	 csr_rd_access_1( {ADDR_CFG_MODE_0, 2'd0});
	 csr_rd_access_1( {ADDR_CFG_MODE_1, 2'd0});
	 csr_rd_access_1( {ADDR_CFG_MODE_2, 2'd0});
	 csr_rd_access_1( {ADDR_CFG_MODE_3, 2'd0});
	  $display(" \n ");
      end
   endtask 

   //-----------------------------------------------------------------------------------------
   // csr read counters from exd_0
   task csr_rd_cnt;
      begin
	 $display("**** Read Counter Registers **** ");
	 csr_rd_access( {ADDR_TX_SOP_CNT, 2'd0});
         csr_rd_access( {ADDR_TX_EOP_CNT, 2'd0});
         csr_rd_access( {ADDR_TX_PKT_CNT, 2'd0});
	 csr_rd_access( {ADDR_RX_CRC_OK_CNT, 2'd0});
         csr_rd_access( {ADDR_RX_CRC_ERR_CNT, 2'd0});
         csr_rd_access( {ADDR_RX_SOP_CNT, 2'd0});
         csr_rd_access( {ADDR_RX_EOP_CNT, 2'd0});
         csr_rd_access( {ADDR_RX_PKT_CNT, 2'd0});
          $display(" \n ");
      end
   endtask // csr_rd_cnt

    //-----------------------------------------------------------------------------------------
    // csr read counters from exd_1
    task csr_rd_cnt_1;
	begin
	    $display("**** Dev_1 Read Counter Registers **** ");
	    csr_rd_access_1( {ADDR_RX_SOP_CNT, 2'd0});
            csr_rd_access_1( {ADDR_RX_EOP_CNT, 2'd0});
            csr_rd_access_1( {ADDR_RX_PKT_CNT, 2'd0});
	    csr_rd_access_1( {ADDR_RX_CRC_OK_CNT, 2'd0});
            csr_rd_access_1( {ADDR_RX_CRC_ERR_CNT, 2'd0});
            
	    csr_rd_access_1( {ADDR_TX_SOP_CNT, 2'd0});
            csr_rd_access_1( {ADDR_TX_EOP_CNT, 2'd0});
            csr_rd_access_1( {ADDR_TX_PKT_CNT, 2'd0});
	    
         $display(" \n ");
	end
    endtask 

    //-----------------------------------------------------------------------------------------
    // csr read counters from exd_0
    task csr_rd_cnt_exp;
	input [31:0] cnt_exp;
	input [31:0] err_cnt_exp;
	
	begin
	    $display("**** Read Counters Stats **** ");
	    csr_rd_access_exp( {ADDR_TX_SOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp( {ADDR_TX_EOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp( {ADDR_TX_PKT_CNT, 2'd0},     cnt_exp);
	    csr_rd_access_exp( {ADDR_RX_CRC_OK_CNT, 2'd0},  cnt_exp);
            csr_rd_access_exp( {ADDR_RX_CRC_ERR_CNT, 2'd0}, err_cnt_exp);
            csr_rd_access_exp( {ADDR_RX_SOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp( {ADDR_RX_EOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp( {ADDR_RX_PKT_CNT, 2'd0},     cnt_exp);
	    repeat (10) @(posedge clk_avmm);
            $display(" \n ");
	end
    endtask // csr_rd_cnt_exp

    //-----------------------------------------------------------------------------------------
    // csr read counters from exd_1
    task csr_rd_cnt_exp_1;
	input [31:0] cnt_exp;
	input [31:0] err_cnt_exp;
	
	begin
	    $display("**** Dev_1 Read Counters Stats **** ");
	    
            csr_rd_access_exp_1( {ADDR_RX_SOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp_1( {ADDR_RX_EOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp_1( {ADDR_RX_PKT_CNT, 2'd0},     cnt_exp);
	    csr_rd_access_exp_1( {ADDR_RX_CRC_OK_CNT, 2'd0},  cnt_exp);
            csr_rd_access_exp_1( {ADDR_RX_CRC_ERR_CNT, 2'd0}, err_cnt_exp);
	    csr_rd_access_exp_1( {ADDR_TX_SOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp_1( {ADDR_TX_EOP_CNT, 2'd0},     cnt_exp);
            csr_rd_access_exp_1( {ADDR_TX_PKT_CNT, 2'd0},     cnt_exp);
            $display(" \n ");
	end
    endtask
    

   //-----------------------------------------------------------------------------------------
   // csr read to exd_0
   task csr_rd_access;
      input [9:0] addr;      

      begin
	 wait (tb_top.exd.waitrequest === 0)

	   @(posedge clk_avmm) begin
	      read      <= '1;
	      address   <= addr;
	      write     <= '0;
	      writedata <= '0;
	   end
	 
	 repeat (1) @(posedge clk_avmm);
	 #10;
	 
	 wait (tb_top.exd.waitrequest === 1)
	   @(posedge clk_avmm) begin
	      read      <= '0;
	      write     <= '0;
	   end
	 repeat (4) @(posedge clk_avmm);
      end
   endtask // csr_rd_access

   //-----------------------------------------------------------------------------------------
   // csr read to exd_1
   task csr_rd_access_1;
      input [9:0] addr;      

      begin
	 wait (tb_top.exd_1.waitrequest === 0)

	   @(posedge clk_avmm) begin
	      read_1      <= '1;
	      address_1   <= addr;
	      write_1     <= '0;
	      writedata_1 <= '0;
	   end
	 
	 repeat (1) @(posedge clk_avmm);
	 #10;
	 
	 wait (tb_top.exd_1.waitrequest === 1)
	   @(posedge clk_avmm) begin
	      read_1      <= '0;
	      write_1     <= '0;
	   end
	 repeat (4) @(posedge clk_avmm);
      end
   endtask

   //-----------------------------------------------------------------------------------------
   // csr read to exd_0
   task csr_rd_access_exp;
      input [9:0] addr;   
      input [31:0] exp_rdata;
   
      begin
	 test_fail = '0;
	 
	 wait (tb_top.exd.waitrequest === 0)

	   @(posedge clk_avmm) begin
	      read      <= '1;
	      address   <= addr;
	      write     <= '0;
	      writedata <= '0;
	   end
	 
	 repeat (1) @(posedge clk_avmm);
	 #10;
	 
	 wait (tb_top.exd.waitrequest === 1)	 
	   @(posedge clk_avmm) begin
	      read      <= '0;
	      write     <= '0;
	   end

	 #10;
	 wait (tb_top.exd.readdatavalid === 1)
	   if (readdata !== exp_rdata) begin
	     $display("RD ERROR: Expected data = %0d; Actual data = %0d;", exp_rdata, readdata);
	      test_fail = '1;
	       repeat (100) @(posedge clk_avmm);
	       $finish;
	       
	   end

	 repeat (4) @(posedge clk_avmm);
      end
      
   endtask // csr_rd_access

   //-----------------------------------------------------------------------------------------
   // csr read to exd_1
   task csr_rd_access_exp_1;
      input [9:0] addr;   
      input [31:0] exp_rdata;
   
      begin
	 test_fail_1 = '0;
	 
	 wait (tb_top.exd_1.waitrequest === 0)

	   @(posedge clk_avmm) begin
	      read_1      <= '1;
	      address_1   <= addr;
	      write_1     <= '0;
	      writedata_1 <= '0;
	   end
	 
	 repeat (1) @(posedge clk_avmm);
	 #10;
	 
	 wait (tb_top.exd_1.waitrequest === 1)	 
	   @(posedge clk_avmm) begin
	      read_1      <= '0;
	      write_1     <= '0;
	   end

	 #10;
	 wait (tb_top.exd_1.readdatavalid === 1)
	   if (readdata_1 !== exp_rdata) begin
	     $display("Dev_1 RD ERROR: Expected data = %0d; Actual data = %0d;", exp_rdata, readdata_1);
	      test_fail_1 = '1;
	   end

	 repeat (4) @(posedge clk_avmm);
      end
      
   endtask // csr_rd_access_exp
   
   //-----------------------------------------------------------------------------------------
   // csr write to exd_0
   task csr_wr_access;
      input [9:0] addr;
      input [31:0] wdata;

      begin
	 wait (tb_top.exd.waitrequest === 0)

	   @(posedge clk_avmm) begin
	      write      <= '1;
	      read       <= '0;
	      address    <= addr;
	      writedata  <= wdata;
	   end
	 
	 repeat (1) @(posedge clk_avmm);
	 #10;
	 
	 wait (tb_top.exd.waitrequest === 1)
	   @(posedge clk_avmm) begin
	      read      <= '0;
	      write     <= '0;
	   end

	 repeat (4) @(posedge clk_avmm);
      end      
   endtask // csr_wr_access

   //-----------------------------------------------------------------------------------------
   // csr write to exd_1
   task csr_wr_access_1;
      input [9:0] addr;
      input [31:0] wdata;

      begin
	 wait (tb_top.exd_1.waitrequest === 0)

	   @(posedge clk_avmm) begin
	      write_1      <= '1;
	      read_1       <= '0;
	      address_1    <= addr;
	      writedata_1  <= wdata;
	   end
	 
	 repeat (1) @(posedge clk_avmm);
	 #10;
	 
	 wait (tb_top.exd_1.waitrequest === 1)
	   @(posedge clk_avmm) begin
	      read_1      <= '0;
	      write_1     <= '0;
	   end

	 repeat (4) @(posedge clk_avmm);
      end      
   endtask


   always_comb begin
      // rx2tx loopback enable
      if (rx2tx_lb_en) begin
	 rx_mii_d = tx_mii_d_1;
	 rx_mii_c = tx_mii_c_1;
	 rx_mii_sync = tx_mii_sync_1;
	 rx_mii_vld  = tx_mii_vld_1;
	 rx_mii_am = tx_mii_am_1;
	 
      end
      // normal op:  loopback at tx2rx of exd_0
      else begin
	 rx_mii_d = tx_mii_d;
	 rx_mii_c = tx_mii_c;
	 rx_mii_sync = tx_mii_sync;
	 rx_mii_vld  = tx_mii_vld;
	 rx_mii_am = tx_mii_am;
      end
   end // always_comb
   
   // example design
   exd exd
     (// inputs
      .tclk (clkf), 
      .clk (clk),
      .clk_avmm (clk_avmm),
      .rst (rst),

      //------------------------------------------------------------------------------------
      // csr interface
      // inputs
      .address (address),
      .writedata (writedata),
      .read (read),
      .write (write),
      
      // outputs
      .readdata (readdata),
      .readdatavalid (readdatavalid),
      .waitrequest (waitrequest),

      //--------------------------------------------------------------------------------------
      // tmii
      // inputs
      .tx_mii_ready ('1),

      // outputs
      .tx_mii_d (tx_mii_d),
      .tx_mii_c (tx_mii_c),
      .tx_mii_sync (tx_mii_sync),
      .tx_mii_vld (tx_mii_vld),
      .tx_mii_am (tx_mii_am),
      //--------------------------------------------------------------------------------------

      //--------------------------------------------------------------------------------------
      // rmii
      .rx_mii_d (rx_mii_d),
      .rx_mii_c (rx_mii_c),
      .rx_mii_sync (rx_mii_sync),
      .rx_mii_vld (rx_mii_vld),
      .rx_mii_am (rx_mii_am)

      );

   

    // example design
   exd exd_1
     (// inputs
      .tclk (clkf), 
      .clk (clk),
      .clk_avmm (clk_avmm),
      .rst (rst),

      //------------------------------------------------------------------------------------
      // csr interface
      // inputs
      .address (address_1),
      .writedata (writedata_1),
      .read (read_1),
      .write (write_1),
      
      // outputs
      .readdata (readdata_1),
      .readdatavalid (readdatavalid_1),
      .waitrequest (waitrequest_1),

      //--------------------------------------------------------------------------------------
      // tmii
      // inputs
      .tx_mii_ready ('1),

      // outputs
      // output to exd_0 if rx2tx_lb_en=1
      .tx_mii_d (tx_mii_d_1),
      .tx_mii_c (tx_mii_c_1),
      .tx_mii_sync (tx_mii_sync_1),
      .tx_mii_vld (tx_mii_vld_1),
      .tx_mii_am (tx_mii_am_1),
      //--------------------------------------------------------------------------------------

      //--------------------------------------------------------------------------------------
      // rmii
      // connect from exd_0
      .rx_mii_d (tx_mii_d),
      .rx_mii_c (tx_mii_c),
      .rx_mii_sync (tx_mii_sync),
      .rx_mii_vld (tx_mii_vld),
      .rx_mii_am (tx_mii_am)

      );

  
   
    always_comb begin
	cfg_mode_reg_0        = readdata;
	cfg_mode_reg_1        = readdata;
	cfg_mode_reg_2        = readdata;
	cfg_mode_reg_3        = readdata;
	cfg_start_pkt_gen_reg = readdata;
	cfg_xfer_pkt_reg      = readdata;
	cnt_reg               = readdata;
	mem_access_ctl_reg    = readdata;
	mem_misc_reg          = readdata;
	cfg_data_reg          = readdata;      
   end
   
    always_ff @(posedge clk_avmm) begin
	if (readdatavalid) begin
	    readdata_reg <= readdata;
	    
	    case (address[9:2])
		ADDR_CFG_MODE_0: begin
		    $display("RD_REG: cfg_mode_reg_0: init_done=%0d; "
			     ,cfg_mode_reg_0.init_done );
		    
		    case (cfg_mode_reg_0.mode_op)
			MODE_OP_e'(MODE_PCS): $display("RD_REG: cfg_mode_reg_0: mode_op=MODE_PCS;");
			MODE_OP_e'(MODE_FLEXE): $display("RD_REG: cfg_mode_reg_0: mode_op=MODE_FLEXE; ");
			MODE_OP_e'(MODE_OTN): $display("RD_REG: cfg_mode_reg_0: mode_op=MODE_OTN;");
		    endcase
		    case (cfg_mode_reg_0.mode)
			MODE_e'(MODE_10_25G) : $display("RD_REG: cfg_mode_reg_0: mode=MODE_10_25G;");
			MODE_e'(MODE_40_50G) : $display("RD_REG: cfg_mode_reg_0: mode=MODE_40_50G;");
			MODE_e'(MODE_100G) : $display("RD_REG: cfg_mode_reg_0: mode=MODE_100G;");
			MODE_e'(MODE_200G) : $display("RD_REG: cfg_mode_reg_0: mode=MODE_200G;");
			MODE_e'(MODE_400G) : $display("RD_REG: cfg_mode_reg_0: mode=MODE_400G;");
		    endcase
		    case (cfg_mode_reg_0.pkt_len_mode)
			PKT_LEN_MODE_e'(FIX_PKT_LEN) : $display("RD_REG: cfg_mode_reg_0: pkt_len_mode=FIX_PKT_LEN;");
			PKT_LEN_MODE_e'(INC_PKT_LEN) : $display("RD_REG: cfg_mode_reg_0: pkt_len_mode=INC_PKT_LEN;");
			PKT_LEN_MODE_e'(RND_PKT_LEN) : $display("RD_REG: cfg_mode_reg_0: pkt_len_mode=RND_PKT_LEN;");
		    endcase
		    case (cfg_mode_reg_0.pat_mode)
			DAT_PAT_MODE_e'(FIX_DAT_PAT): $display("RD_REG: cfg_mode_reg_0: pat_mode=FIX_DAT_PAT;");
			DAT_PAT_MODE_e'(INC_DAT_PAT): $display("RD_REG: cfg_mode_reg_0: pat_mode=INC_DAT_PAT;");
			DAT_PAT_MODE_e'(RND_DAT_PAT): $display("RD_REG: cfg_mode_reg_0: pat_mode=RND_DAT_PAT;");
		    endcase
		    
	            
		    $display("RD_REG: cfg_mode_reg_0: cont_xfer_mode=%0d; disable_am_ins=%0d; "
			     ,cfg_mode_reg_0.cont_xfer_mode
			     ,cfg_mode_reg_0.disable_am_ins
			     );
		    
		    $display("RD_REG: cfg_mode_reg_0: rx2tx_lb=%0d; ipg_dly=%0d; tmii_rdy_fix_dly=%0d;"                 
			     ,cfg_mode_reg_0.rx2tx_lb
			     ,cfg_mode_reg_0.ipg_dly
			     ,cfg_mode_reg_0.tmii_rdy_fix_dly);
		    
		end // case: ADDR_CFG_MODE_0
		ADDR_CFG_MODE_1: begin
		    $display("RD_REG: cfg_mode_reg_1: no_of_inc_bytes=%0d;",cfg_mode_reg_1.no_of_inc_bytes);
		    
		    $display("RD_REG: cfg_mode_reg_1: fix_pattern=0x%0h; fix_pkt_len=%0d;"
			     ,cfg_mode_reg_1.fix_pattern
			     ,cfg_mode_reg_1.fix_pkt_len);
		end
		ADDR_CFG_MODE_2: begin
		    $display("RD_REG: cfg_mode_reg_2: no_of_pkt_gen=%0d; no_of_xfer_pkt=%0d;"
			     ,cfg_mode_reg_2.no_of_pkt_gen
			     ,cfg_mode_reg_2.no_of_xfer_pkt);
		end
		ADDR_CFG_MODE_3: begin
		    $display("RD_REG: cfg_mode_reg_3: am_ins_period=%0d; am_ins_cyc=%0d;"
			     ,cfg_mode_reg_3.am_ins_period
			     ,cfg_mode_reg_3.am_ins_cyc);
		end
		ADDR_CFG_START_PKT_GEN: begin
		    $display("RD_REG: cfg_start_pkt_gen_reg : start_pkt_gen=%0d; pkt_gen_done_stat=%0d;"
			     ,cfg_start_pkt_gen_reg.start_pkt_gen
			     ,cfg_start_pkt_gen_reg.pkt_gen_done_stat);
		end
		ADDR_CFG_START_XFER_PKT: begin
		    $display("RD_REG:  cfg_xfer_pkt_reg: start_xfer_pkt=%0d; pkt_xfer_done_stat=%0d;"
			     ,cfg_xfer_pkt_reg.start_xfer_pkt
			     ,cfg_xfer_pkt_reg.pkt_xfer_done_stat);
		end
		ADDR_TX_SOP_CNT: $display("RD_REG:  tx_sop_cnt=%0d; ",cnt_reg );
		ADDR_TX_EOP_CNT: $display("RD_REG:  tx_eop_cnt=%0d; ",cnt_reg );
		ADDR_TX_PKT_CNT: $display("RD_REG:  tx_pkt_cnt=%0d; ",cnt_reg );
		ADDR_RX_SOP_CNT: $display("RD_REG:  rx_sop_cnt=%0d; ",cnt_reg );
		ADDR_RX_EOP_CNT: $display("RD_REG:  rx_eop_cnt=%0d; ",cnt_reg );
		ADDR_RX_PKT_CNT: $display("RD_REG:  rx_pkt_cnt=%0d; ",cnt_reg );
		ADDR_RX_CRC_OK_CNT: $display("RD_REG:  rx_crc_ok_cnt=%0d; ",cnt_reg );
		ADDR_RX_CRC_ERR_CNT: $display("RD_REG:  rx_crc_err_cnt=%0d; ",cnt_reg );
		ADDR_MEM_MISC:  $display("RD_REG:  mem_misc: sop=%0h; terminate=%0d"
                                    ,mem_misc_reg.sop
                                    ,mem_misc_reg.terminate  );
		ADDR_MEM_ACCESS: $display("RD_REG:  mem_acc_ctl: addr=%0h; rd_req=%0d; wr_req=%0d; "
                                    ,mem_access_ctl_reg.addr_req
				    ,mem_access_ctl_reg.rd_req
                                    ,mem_access_ctl_reg.wr_req);
		default: begin
		    if (address[9:8] == 2'b10) begin
			$display("RD_REG:  mem_data_%0d==%0h; ",address[7:2], cfg_data_reg );
		    end
		    else if (address[9:8] == 2'b11) begin
			$display("RD_REG:  mem_ctl_%0d==%0h; ",address[4:2], cfg_data_reg );
		    end
		    
		end
	 endcase 

      end // if (readdatavalid)
   end // always_ff @ (posedge clk_avmm)


   always_comb begin
      cfg_mode_1_reg_0        = readdata_1;
      cfg_mode_1_reg_1        = readdata_1;
      cfg_mode_1_reg_2        = readdata_1;
      cfg_mode_1_reg_3        = readdata_1;
      cfg_start_pkt_gen_1_reg = readdata_1;
      cfg_xfer_pkt_1_reg      = readdata_1;
      cnt_1_reg               = readdata_1;     
   end
   
   always_ff @(posedge clk_avmm) begin
      if (readdatavalid_1) begin
	 readdata_1_reg <= readdata_1;
	 
	 case (address_1[9:2])
	     
	   ADDR_CFG_MODE_0: begin
	      $display("Dev_1 RD_REG: cfg_mode_reg_0: init_done=%0d; "
                 ,cfg_mode_1_reg_0.init_done );
	       
	      case (cfg_mode_1_reg_0.mode_op)
		MODE_OP_e'(MODE_PCS): $display("Dev_1 RD_REG: cfg_mode_reg_0: mode_op=MODE_PCS;");
                MODE_OP_e'(MODE_FLEXE): $display("Dev_1 RD_REG: cfg_mode_reg_0: mode_op=MODE_FLEXE; ");
                MODE_OP_e'(MODE_OTN): $display("Dev_1 RD_REG: cfg_mode_reg_0: mode_op=MODE_OTN;");
	      endcase
              case (cfg_mode_1_reg_0.mode)
		MODE_e'(MODE_10_25G) : $display("Dev_1 RD_REG: cfg_mode_reg_0: mode=MODE_10_25G;");
		MODE_e'(MODE_40_50G) : $display("Dev_1 RD_REG: cfg_mode_reg_0: mode=MODE_40_50G;");
		MODE_e'(MODE_100G) : $display("Dev_1 RD_REG: cfg_mode_reg_0: mode=MODE_100G;");
		MODE_e'(MODE_200G) : $display("Dev_1 RD_REG: cfg_mode_reg_0: mode=MODE_200G;");
		MODE_e'(MODE_400G) : $display("Dev_1 RD_REG: cfg_mode_reg_0: mode=MODE_400G;");
	      endcase
	      case (cfg_mode_1_reg_0.pkt_len_mode)
	        PKT_LEN_MODE_e'(FIX_PKT_LEN) : $display("Dev_1 RD_REG: cfg_mode_reg_0: pkt_len_mode=FIX_PKT_LEN;");
		PKT_LEN_MODE_e'(INC_PKT_LEN) : $display("Dev_1 RD_REG: cfg_mode_reg_0: pkt_len_mode=INC_PKT_LEN;");
		PKT_LEN_MODE_e'(RND_PKT_LEN) : $display("Dev_1 RD_REG: cfg_mode_reg_0: pkt_len_mode=RND_PKT_LEN;");
	      endcase
	      case (cfg_mode_1_reg_0.pat_mode)
		DAT_PAT_MODE_e'(FIX_DAT_PAT): $display("Dev_1 RD_REG: cfg_mode_reg_0: pat_mode=FIX_DAT_PAT;");
		DAT_PAT_MODE_e'(INC_DAT_PAT): $display("Dev_1 RD_REG: cfg_mode_reg_0: pat_mode=INC_DAT_PAT;");
		DAT_PAT_MODE_e'(RND_DAT_PAT): $display("Dev_1 RD_REG: cfg_mode_reg_0: pat_mode=RND_DAT_PAT;");
	      endcase
		
	          
	      $display("Dev_1 RD_REG: cfg_mode_reg_0: cont_xfer_mode=%0d; disable_am_ins=%0d;"
                 ,cfg_mode_1_reg_0.cont_xfer_mode
		 ,cfg_mode_1_reg_0.disable_am_ins
	         );

	      $display("Dev_1 RD_REG: cfg_mode_reg_0: rx2tx_lb=%0d; ipg_dly=%0d; tmii_rdy_fix_dly=%0d;"
                 ,cfg_mode_1_reg_0.rx2tx_lb
		 ,cfg_mode_1_reg_0.ipg_dly
                 ,cfg_mode_1_reg_0.tmii_rdy_fix_dly);
	   end // case: ADDR_CFG_MODE_0
	   ADDR_CFG_MODE_1: begin
	      $display("Dev_1 RD_REG: cfg_mode_reg_1: no_of_inc_bytes=%0d; ",cfg_mode_1_reg_1.no_of_inc_bytes);
	      
	      $display("Dev_1 RD_REG: cfg_mode_reg_1: fix_pattern=0x%0h; fix_pkt_len=%0d;"
                 ,cfg_mode_1_reg_1.fix_pattern
                 ,cfg_mode_1_reg_1.fix_pkt_len);
	   end
	   ADDR_CFG_MODE_2: begin
	      $display("Dev_1 RD_REG: cfg_mode_reg_2: no_of_pkt_gen=%0d; no_of_xfer_pkt=%0d;"
                ,cfg_mode_1_reg_2.no_of_pkt_gen
		,cfg_mode_1_reg_2.no_of_xfer_pkt);
	   end
	   ADDR_CFG_MODE_3: begin
	      $display("Dev_1 RD_REG: cfg_mode_reg_3: am_ins_period=%0d; am_ins_cyc=%0d;"
                ,cfg_mode_1_reg_3.am_ins_period
		,cfg_mode_1_reg_3.am_ins_cyc);
	   end
	   ADDR_CFG_START_PKT_GEN: begin
	      $display("Dev_1 RD_REG: cfg_start_pkt_gen_reg : start_pkt_gen=%0d; pkt_gen_done_stat=%0d;"
                ,cfg_start_pkt_gen_1_reg.start_pkt_gen
		,cfg_start_pkt_gen_1_reg.pkt_gen_done_stat);
	   end
	   ADDR_CFG_START_XFER_PKT: begin
	      $display("Dev_1 RD_REG:  cfg_xfer_pkt_reg: start_xfer_pkt=%0d; pkt_xfer_done_stat=%0d;"
                ,cfg_xfer_pkt_1_reg.start_xfer_pkt
		,cfg_xfer_pkt_1_reg.pkt_xfer_done_stat);
	   end
	   ADDR_TX_SOP_CNT: $display("Dev_1 RD_REG:  tx_sop_cnt=%0d; ",cnt_1_reg );
	   ADDR_TX_EOP_CNT: $display("Dev_1 RD_REG:  tx_eop_cnt=%0d; ",cnt_1_reg );
           ADDR_TX_PKT_CNT: $display("Dev_1 RD_REG:  tx_pkt_cnt=%0d; ",cnt_1_reg );
	   ADDR_RX_SOP_CNT: $display("Dev_1 RD_REG:  rx_sop_cnt=%0d; ",cnt_1_reg );
	   ADDR_RX_EOP_CNT: $display("Dev_1 RD_REG:  rx_eop_cnt=%0d; ",cnt_1_reg );
           ADDR_RX_PKT_CNT: $display("Dev_1 RD_REG:  rx_pkt_cnt=%0d; ",cnt_1_reg );
	   ADDR_RX_CRC_OK_CNT: $display("Dev_1 RD_REG:  rx_crc_ok_cnt=%0d; ",cnt_1_reg );
	   ADDR_RX_CRC_ERR_CNT: $display("Dev_1 RD_REG:  rx_crc_err_cnt=%0d; ",cnt_1_reg );
	   
	 endcase 

      end // if (readdatavalid)
   end
   
endmodule // tb
