# (C) 2001-2023 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


# set the number of IP instances
set ip_num  1
set LANE_NUM 4
set ENABLE_DL_GUI 0

# Setting 100Mhz reconfig clock for ANLT and 250Mhz for base IP and PTP
create_clock -name {reconfig_clk} -period 4.000 -waveform { 0.000 2.000 } [get_ports {i_reconfig_clk}]

create_clock -name {altera_reserved_tck} -period 40 [get_ports {altera_reserved_tck}]


for {set i 0} {$i < $ip_num} {incr i} {
    set ip_inst "IP_INST\[$i\]."

    set "TXCLKOUT_CHANNEL{i}"   [get_clocks "${ip_inst}hw_ip_top|dut|eth_f_0|tx_clkout|ch*"]
    set "TXCLKOUT2_CHANNEL{i}"  [get_clocks "${ip_inst}hw_ip_top|dut|eth_f_0|tx_clkout2|ch*"]
    set "RXCLKOUT_CHANNEL{i}"   [get_clocks "${ip_inst}hw_ip_top|dut|eth_f_0|rx_clkout|ch*"]
    set "RXCLKOUT2_CHANNEL{i}"  [get_clocks "${ip_inst}hw_ip_top|dut|eth_f_0|rx_clkout2|ch*"]
}

set RECONFIG_CLK [get_clocks reconfig_clk]

# From Timequest cookbook
set_clock_groups -asynchronous -group [get_clocks altera_reserved_tck]


# From "AV SoC Golden Hardware Reference Design"
set_input_delay -clock altera_reserved_tck -clock_fall 3 [get_ports altera_reserved_tdi] -add_delay
set_input_delay -clock altera_reserved_tck -clock_fall 3 [get_ports altera_reserved_tms] -add_delay
# set_input_delay -clock altera_reserved_tck -clock_fall 3 [get_ports altera_reserved_ntrst]
set_output_delay -clock altera_reserved_tck 3 [get_ports altera_reserved_tdo] -add_delay

proc eth_f_constraint_net_delay {from_reg to_reg max_net_delay {check_exist 0} {get_pins 1} {set_skew_constraint 1} {set_mstable 1} {set_no_synchronizer 0}} {
    
    # Check for instances
    set inst [get_registers -nowarn ${to_reg}]
    
    # Check number of instances
    set inst_num [llength [query_collection -report -all $inst]]
    if {$inst_num > 0} {
        # Uncomment line below for debug purpose
        #puts "${inst_num} ${to_reg} instance(s) found"
    } else {
        # Uncomment line below for debug purpose
        #puts "No ${to_reg} instance found"
    }

  if {($set_skew_constraint == 0)} {   
    if {($check_exist == 0) || ($inst_num > 0)} {
        if { [string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
            if {$set_no_synchronizer == 1} {
                set_max_delay -no_synchronizer -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            } else {
                set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            }
            set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -200ns
        } else {
            if {$get_pins == 0} {
                set_net_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -max $max_net_delay
            } else {
                set_net_delay -from [get_pins -compatibility_mode ${from_reg}|q] -to [get_registers ${to_reg}] -max $max_net_delay
            }
            
            # Relax the fitter effort
            if {$set_no_synchronizer == 1} {
                set_max_delay -no_synchronizer -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            } else {
                set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            }
            set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -200ns
        }
    }
  } else {   
        #set skew and min/max delay for EFIFO
        # control skew for bits
        set_max_skew -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -get_skew_value_from_clock_period src_clock_period -skew_value_multiplier 0.8
        # path delay (exception for net delay)
        if { ![string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
            set_net_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
        }
        #relax setup and hold calculation
        if {$set_no_synchronizer == 1} {
            set_max_delay -no_synchronizer -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 100
        } else {
            set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 100
        }
        set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -100		
  }
	
    #set Meta stability for EFIFO 
    if {($set_mstable == 1)} {
		if { ![string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
			set_net_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
		}
    }
}

for {set i 0} {$i < $ip_num} {incr i} {
    set ip_inst "IP_INST\[$i\]."

    # packet client resets, packet client csr clear signals
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|pkt_client_rst_n" \
                                * \
                                2.2ns 1 0 0 0 0
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|packet_client_csr|reg_00\[*\]" \
                                * \
                                2.2ns 1 0 0 0 0
    # packet client csr counters
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|loopback_client|stat_counter|stat_cnt_vld" \
                                "${ip_inst}hw_ip_top|packet_client_top|packet_client_csr|*|cnt_vld_sync" \
                                2.2ns 1 0 0 0 0
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|loopback_client|stat_counter|stat_*_cnt\[*\]" \
                                "${ip_inst}hw_ip_top|packet_client_top|packet_client_csr|u_*_cnt|cnt_*\[*\]" \
                                2.2ns 1 0 0 0 0
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|packet_client_tx_if|stat_counter|stat_cnt_vld" \
                                "${ip_inst}hw_ip_top|packet_client_top|packet_client_csr|*|cnt_vld_sync" \
                                2.2ns 1 0 0 0 0
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|packet_client_tx_if|stat_counter|stat_*_cnt\[*\]" \
                                "${ip_inst}hw_ip_top|packet_client_top|packet_client_csr|u_*_cnt|cnt_*\[*\]" \
                                2.2ns 1 0 0 0 0
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|loopback_client|eth_f_loopback_fifo|o_wr_full_err" \
                                "${ip_inst}hw_ip_top|packet_client_top|packet_client_csr|loopback_fifo_wr_full_err" \
                                2.2ns 1 0 0 0 0
    eth_f_constraint_net_delay  "${ip_inst}hw_ip_top|packet_client_top|packet_client_top|pkt_gen_top|pkt_gen_pingpong_bf|full*" \
                                "${ip_inst}hw_ip_top|packet_client_top|packet_client_top|pkt_gen_top|pkt_gen_rom|pkt_altsyncram|auto_generated|altera_syncram_impl1|ram_block*" \
                                2.2ns 1 0 0 0 0
  





}


#OTN Variants DA warnings constriants #RES-50003





