# (C) 2001-2023 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source [file join [file dirname [info script]] "altera/sval_top/main_100G.tcl"]
source [file join [file dirname [info script]] "altera/sval_top/main_misc.tcl"]
 
# Function to run a simple series of tests
 
proc run_test_without_loopback {} {
    global CLIENT_INT_GUI
    global BASE_EHIPRegisters
    global CLIENT_BASE
    global PCS_Status_chk
    global BASE_SoftCSRs
    global ADDR_PCS_STATUS
    global ADDR_LINK_FAULT
    global LINK_Status_chk
    global RX_REG_CRCERR_LO
    global RX_REG_OCT_OK_LO

    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
 
        puts "--- Turning off packet generation ----"
        puts "--------------------------------------\n"
        stop_pkt_gen $ip_inst_base_addr
 
        #puts "--------- Enabling loopback ----------"
        #puts "--------------------------------------\n"
        #loop_on
 
        puts "--- Wait for RX clock to settle... ---"
        puts "--------------------------------------\n"
        sleep 2
 
        puts "-------- Printing PHY status ---------"
        puts "--------------------------------------\n"
        chkphy_status $ip_inst_base_addr

        set PCS_Status_chk [reg_read [expr $ip_inst_base_addr + $BASE_SoftCSRs] $ADDR_PCS_STATUS] 
        set LINK_Status_chk [reg_read [expr $ip_inst_base_addr + $BASE_SoftCSRs] $ADDR_LINK_FAULT]
 
        if {[expr $PCS_Status_chk  & 0x8]==0x8 && $LINK_Status_chk == 0x0} {
        
        if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "---- Clearing MAC stats counters -----"
            puts "--------------------------------------\n"
            clear_all_stats $ip_inst_base_addr
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "---- Clearing packet counters -----"
            puts "--------------------------------------\n"
            clear_pkt_counters $ip_inst_base_addr
        }               

    if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "--- Initialize PKT ROM Read address for IP_INST\[$inst_num\]----"
            puts "--------------------------------------\n"
            init_packet_rom "100G" $ip_inst_base_addr
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "-------------------------------------"
            puts "--------------------------------------\n"
        }        
        puts "--------- Sending packets... ---------"
        puts "--------------------------------------\n"
        start_pkt_gen $ip_inst_base_addr
        sleep 1
        stop_pkt_gen $ip_inst_base_addr
 
        if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "----- Reading MAC stats counters -----"
            puts "--------------------------------------\n"
            chkmac_stats $ip_inst_base_addr


        set tx_65_127_byte_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x860]
        set rx_65_127_byte_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x994]

        set tx_frame_start [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x8D8]
        set rx_frame_start [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0xA0C]

        set tx_multicast_data_ok_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x898]
        set rx_multicast_data_ok_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x9CC]
        
        set RX_REG_CRCERR_LO [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x944]
        set TX_REG_CRCERR_LO [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x810]
        
        set TX_REG_OCT_OK_LO [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x900]
        set RX_REG_OCT_OK_LO_1 [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0xA34]
       
      if { ($tx_65_127_byte_frames == 16) && ($rx_65_127_byte_frames == 16) &&  ($tx_frame_start == 16 && $rx_frame_start == 16) && (!$RX_REG_CRCERR_LO) &&  (!$TX_REG_CRCERR_LO) && ($TX_REG_OCT_OK_LO == 1152 && $RX_REG_OCT_OK_LO_1 == 1152)  } {
        
        puts "--------------------------------------\n"
        puts  "run_test:pass"
        puts "--------------------------------------\n"
        } else {
        puts "--------------------------------------\n"
        puts  "run_test:fail"
        puts "--------------------------------------\n"
        }
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "----- Reading packet counters -----"
            puts "--------------------------------------\n"
            chk_pkt_counters $ip_inst_base_addr

         set Tx_Start_pkt_Counter [reg_read 0x18 [expr $ip_inst_base_addr + $CLIENT_BASE]]
         set Rx_Start_pkt_Counter [reg_read 0x30 [expr $ip_inst_base_addr + $CLIENT_BASE]]

         set Tx_End_pkt_Counter [reg_read 0x20 [expr $ip_inst_base_addr + $CLIENT_BASE]]
         set Rx_End_pkt_Counter [reg_read 0x38 [expr $ip_inst_base_addr + $CLIENT_BASE]]

        if {($Tx_Start_pkt_Counter == 16 && $Rx_Start_pkt_Counter  == 16 ) && ($Tx_End_pkt_Counter == 16 && $Rx_End_pkt_Counter ==16 )} {
        puts "--------------------------------------\n"
        puts  "run_test:pass"
        puts "--------------------------------------\n"
        } else {
        puts "--------------------------------------\n"
        puts  "run_test:fail"
        puts "--------------------------------------\n"
        }
        }
           } else {
          read_reset_issp
          puts "ERROR:rx_pcs_ready is not asserted or link fault is detected. Expected value from issp reset probe is : 0xff 
"
    }     
        puts "---------------- Done ----------------\n"
    } 
}


proc run_test {} {
    global XCVR_TYPE_GUI
    global LANE_NUM
    global CLIENT_INT_GUI
    global BASE_EHIPRegisters
    global CLIENT_BASE
    global PCS_Status_chk
    global BASE_SoftCSRs
    global ADDR_PCS_STATUS
    global ADDR_LINK_FAULT
    global LINK_Status_chk
    global RX_REG_CRCERR_LO
    global RX_REG_OCT_OK_LO

    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
 
        puts "--- Turning off packet generation ----"
        puts "--------------------------------------\n"
        stop_pkt_gen $ip_inst_base_addr
 
        puts "--------- Enabling loopback ----------"
        puts "--------------------------------------\n"
        if {$XCVR_TYPE_GUI==0} {
            internal_loop_back_ux $::INSTANCE_NUM $ip_inst_base_addr $LANE_NUM 3 2
        } elseif {$XCVR_TYPE_GUI==1} {
            internal_loop_back_bk $::INSTANCE_NUM $LANE_NUM 
        }
 
        puts "--- Wait for RX clock to settle... ---"
        puts "--------------------------------------\n"
        sleep 2
 
        puts "-------- Printing PHY status ---------"
        puts "--------------------------------------\n"
        chkphy_status $ip_inst_base_addr

        set PCS_Status_chk [reg_read [expr $ip_inst_base_addr + $BASE_SoftCSRs] $ADDR_PCS_STATUS] 
        set LINK_Status_chk [reg_read [expr $ip_inst_base_addr + $BASE_SoftCSRs] $ADDR_LINK_FAULT]
 
        if {[expr $PCS_Status_chk  & 0x8]==0x8 && $LINK_Status_chk == 0x0} { 
       
        if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "---- Clearing MAC stats counters -----"
            puts "--------------------------------------\n"
            clear_all_stats $ip_inst_base_addr
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "---- Clearing packet counters -----"
            puts "--------------------------------------\n"
            clear_pkt_counters $ip_inst_base_addr
        }               
       if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "--- Initialize PKT ROM Read address for IP_INST\[$inst_num\]----"
            puts "--------------------------------------\n"
            init_packet_rom "100G" $ip_inst_base_addr
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "-------------------------------------"
            puts "--------------------------------------\n"
        }       
 
        puts "--------- Sending packets... ---------"
        puts "--------------------------------------\n"
        start_pkt_gen $ip_inst_base_addr
        sleep 1
        stop_pkt_gen $ip_inst_base_addr
 
        if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "----- Reading MAC stats counters -----"
            puts "--------------------------------------\n"
            chkmac_stats $ip_inst_base_addr

        set tx_65_127_byte_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x860]
        set rx_65_127_byte_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x994]

        set tx_frame_start [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x8D8]
        set rx_frame_start [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0xA0C]

        set tx_multicast_data_ok_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x898]
        set rx_multicast_data_ok_frames [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x9CC]
        
        set RX_REG_CRCERR_LO [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x944]
        set TX_REG_CRCERR_LO [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x810]
        
        set TX_REG_OCT_OK_LO [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0x900]
        set RX_REG_OCT_OK_LO_1 [reg_read [expr $ip_inst_base_addr + $BASE_EHIPRegisters] 0xA34]

        if { ($tx_65_127_byte_frames == 16) && ($rx_65_127_byte_frames == 16) &&  ($tx_frame_start == 16 && $rx_frame_start == 16) && (!$RX_REG_CRCERR_LO) &&  (!$TX_REG_CRCERR_LO) && ($TX_REG_OCT_OK_LO == 1152 && $RX_REG_OCT_OK_LO_1 == 1152)  } {
        
        puts "--------------------------------------\n"
        puts  "run_test:pass"
        puts "--------------------------------------\n"
        } else {
        puts "--------------------------------------\n"
        puts  "run_test:fail"
        puts "--------------------------------------\n"
        }
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "----- Reading packet counters -----"
            puts "--------------------------------------\n"
            chk_pkt_counters $ip_inst_base_addr

         set Tx_Start_pkt_Counter [reg_read 0x18 [expr $ip_inst_base_addr + $CLIENT_BASE]]
         set Rx_Start_pkt_Counter [reg_read 0x30 [expr $ip_inst_base_addr + $CLIENT_BASE]]

         set Tx_End_pkt_Counter [reg_read 0x20 [expr $ip_inst_base_addr + $CLIENT_BASE]]
         set Rx_End_pkt_Counter [reg_read 0x38 [expr $ip_inst_base_addr + $CLIENT_BASE]]

        if {($Tx_Start_pkt_Counter == 16 && $Rx_Start_pkt_Counter  == 16 ) && ($Tx_End_pkt_Counter == 16 && $Rx_End_pkt_Counter ==16 )} {
        puts "--------------------------------------\n"
        puts  "run_test:pass"
        puts "--------------------------------------\n"
        } else {
        puts "--------------------------------------\n"
        puts  "run_test:fail"
        puts "--------------------------------------\n"
        }
        }
           } else {
          read_reset_issp
          puts "ERROR:rx_pcs_ready is not asserted or link fault is detected. Expected value from issp reset probe is : 0xff 
"
    }         
        puts "---------------- Done ----------------\n"
    } 
}


proc pcs_alignment_check {} {
    global XCVR_TYPE_GUI
    global LANE_NUM
    
    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
 
        puts "--------- Enabling loopback ----------"
        puts "--------------------------------------\n"
                
        if {$XCVR_TYPE_GUI==0} {
            internal_loop_back_ux $::INSTANCE_NUM $ip_inst_base_addr $LANE_NUM 3 2
        } elseif {$XCVR_TYPE_GUI==1} {
            internal_loop_back_bk $::INSTANCE_NUM $LANE_NUM 
        }
 
        puts "--- Wait for RX clock to settle... ---"
        puts "--------------------------------------\n"
        sleep 2
 
        puts "-------- Printing PHY status ---------"
        puts "--------------------------------------\n"
        chkphy_status $ip_inst_base_addr        
    } 
}


proc rxcore_clock_test {} {
    global XCVR_TYPE_GUI
    global LANE_NUM

    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
 
        puts "--------- Enabling loopback ----------"
        puts "--------------------------------------\n"
                
        if {$XCVR_TYPE_GUI==0} {
            internal_loop_back_ux $::INSTANCE_NUM $ip_inst_base_addr $LANE_NUM 3 2
        } elseif {$XCVR_TYPE_GUI==1} {
            internal_loop_back_bk $::INSTANCE_NUM $LANE_NUM  
        }
 
        puts "--- Wait for RX clock to settle... ---"
        puts "--------------------------------------\n"
        sleep 5
 
        puts "-------- Printing PHY status ---------"
        puts "--------------------------------------\n"
        chkphy_status $ip_inst_base_addr        
    }
}


proc txcore_clock_test {} {
    global XCVR_TYPE_GUI
    global LANE_NUM

    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
 
        puts "--------- Enabling loopback ----------"
        puts "--------------------------------------\n"
                
        if {$XCVR_TYPE_GUI==0} {
            internal_loop_back_ux $::INSTANCE_NUM $ip_inst_base_addr $LANE_NUM 3 2
        } elseif {$XCVR_TYPE_GUI==1} {
            internal_loop_back_bk $::INSTANCE_NUM $LANE_NUM  
        }
 
        puts "--- Wait for RX clock to settle... ---"
        puts "--------------------------------------\n"
        sleep 5
 
        puts "-------- Printing PHY status ---------"
        puts "--------------------------------------\n"
        chkphy_status $ip_inst_base_addr        
    }
}


proc los_test {no_of_times} {
    global XCVR_TYPE_GUI
    global LANE_NUM

    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
 
        for {set i 0} {$i < $no_of_times} {incr i} { 
            puts "--------- Disabling loopback ----------"
            puts "--------------------------------------\n"            
                        
            if {$XCVR_TYPE_GUI==0} {
                disable_loop_back_ux $LANE_NUM 3 2
            } elseif {$XCVR_TYPE_GUI==1} {
                disable_loop_back_bk $LANE_NUM 
            }
                        
            sleep 2
                        
            puts "-------- Printing PHY status ---------"
            puts "--------------------------------------\n"
            chkphy_status $ip_inst_base_addr
                        
            puts "--------- Enabling loopback ----------"
            puts "--------------------------------------\n"
                        
            if {$XCVR_TYPE_GUI==0} {
                internal_loop_back_ux $::INSTANCE_NUM $ip_inst_base_addr $LANE_NUM 3 2
            } elseif {$XCVR_TYPE_GUI==1} {
                internal_loop_back_bk $::INSTANCE_NUM $LANE_NUM 
            }

            puts "-------- Printing PHY status ---------"
            puts "--------------------------------------\n"
            chkphy_status $ip_inst_base_addr        
        }
    }
}


proc reset_test {} {
    soft_global_reset
    soft_tx_reset   
    soft_rx_reset 
}


proc soft_global_reset {no_of_times} {
    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
                
        for {set i 0} {$i < $no_of_times} {incr i} {
            puts "--------- Applying global reset ----------"
            puts "--------------------------------------\n"
            reg_write 0x108 0x1
            chkphy_status $ip_inst_base_addr
            puts "--------- Releasing global reset ----------"
            puts "--------------------------------------\n"
            reg_write 0x108 0x0
            chkphy_status $ip_inst_base_addr
        }
    }          
}


proc soft_tx_reset {no_of_times} {
    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
                
        for {set i 0} {$i < $no_of_times} {incr i} {
            puts "--------- Applying TX reset ----------"
            puts "--------------------------------------\n"
            reg_write 0x108 0x2
            chkphy_status $ip_inst_base_addr
            puts "--------- Releasing TX reset ----------"
            puts "--------------------------------------\n"
            reg_write 0x108 0x0
            chkphy_status $ip_inst_base_addr
        }
    }          
}


proc soft_rx_reset {no_of_times} {
    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]] 
                
        for {set i 0} {$i < $no_of_times} {incr i} {
            puts "--------- Applying RX reset ----------"
            puts "--------------------------------------\n"
            reg_write 0x108 0x4
            chkphy_status $ip_inst_base_addr
            puts "--------- Releasing RX reset ----------"
            puts "--------------------------------------\n"
            reg_write 0x108 0x0
            chkphy_status $ip_inst_base_addr
        }
    }          
}


proc multi_reset_test {no_of_times} {
    global CLIENT_INT_GUI

    for {set inst_num 0} {$inst_num < $::INSTANCE_NUM} {incr inst_num} {
        # obtain ip inst base addr offset
        set ip_inst_base_addr [format 0x%X [expr $inst_num << 24]]
                
        for {set i 0} {$i < $no_of_times} {incr i} {
        puts "--------- Applying RX and TX reset $i ----------"
        puts "--------------------------------------\n"
        reg_write 0x108 0x6
        puts "--------- Releasing RX and TX reset ----------"
        puts "--------------------------------------\n"
        reg_write 0x108 0x0
        
        puts "--- Wait for RX clock to settle... ---"
        puts "--------------------------------------\n"
        sleep 2
 
        puts "-------- Printing PHY status ---------"
        puts "--------------------------------------\n"
        chkphy_status $ip_inst_base_addr

        if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "---- Clearing MAC stats counters -----"
            puts "--------------------------------------\n"
            clear_all_stats $ip_inst_base_addr
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "---- Clearing packet counters -----"
            puts "--------------------------------------\n"
            clear_pkt_counters $ip_inst_base_addr
        }               

        puts "--- Initialize PKT ROM Read address for IP_INST\[$inst_num\]----"
        puts "--------------------------------------\n"
        init_packet_rom "100G" $ip_inst_base_addr
 
        puts "--------- Sending packets... ---------"
        puts "--------------------------------------\n"
        start_pkt_gen $ip_inst_base_addr
        sleep 1
        stop_pkt_gen $ip_inst_base_addr
 
        if {$CLIENT_INT_GUI == 0 || $CLIENT_INT_GUI == 1} {
            puts "----- Reading MAC stats counters -----"
            puts "--------------------------------------\n"
            chkmac_stats $ip_inst_base_addr
        } elseif {$CLIENT_INT_GUI == 2 || $CLIENT_INT_GUI == 3 || $CLIENT_INT_GUI == 4} {
            puts "----- Reading packet counters -----"
            puts "--------------------------------------\n"
            chk_pkt_counters $ip_inst_base_addr
        }
        
        puts "---------------- Done ----------------\n"

        }
    }          
}


 

